import json
import textwrap
import unittest

import pytest

from conans.test.utils.tools import TestClient


class PyRequiresExtendTest(unittest.TestCase):

    @pytest.mark.tool_git
    def test_reuse_scm(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class SomeBase(object):
                scm = {"type" : "git",
                       "url" : "somerepo",
                       "revision" : "auto"}

            class MyConanfileBase(SomeBase, ConanFile):
                pass
            """)
        client.init_git_repo({"conanfile.py": conanfile}, branch="my_release")
        client.run("export . base/1.1@user/testing")

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.SomeBase"
            """)
        client.save({"conanfile.py": reuse})
        client.run("export . Pkg/0.1@user/testing")
        client.run("get Pkg/0.1@user/testing")
        self.assertNotIn("scm = base.scm", client.out)
        self.assertIn('scm = {"revision":', client.out)
        self.assertIn('"type": "git",', client.out)
        self.assertIn('"url": "somerepo"', client.out)

    @pytest.mark.tool_git
    def test_reuse_customize_scm(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class SomeBase(object):
                base_repo = "somerepo"
                scm = {"type" : "git",
                       "url" : base_repo,
                       "revision" : "auto"}

            class MyConanfileBase(SomeBase, ConanFile):
                pass
            """)
        client.init_git_repo({"conanfile.py": conanfile}, branch="my_release")
        client.run("export . base/1.1@user/testing")
        client.run("get base/1.1@user/testing")
        self.assertIn('"url": "somerepo"', client.out)

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                base_repo = "other_repo"
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.SomeBase"
                def init(self):
                    self.scm["url"] = self.base_repo
            """)
        client.save({"conanfile.py": reuse})
        client.run("export . Pkg/0.1@user/testing")
        client.run("get Pkg/0.1@user/testing")
        self.assertNotIn("scm = base.scm", client.out)
        self.assertIn('scm = {"revision":', client.out)
        self.assertIn('"type": "git",', client.out)
        self.assertIn('"url": "other_repo"', client.out)

    @pytest.mark.tool_git
    def test_reuse_scm_multiple_conandata(self):
        # https://github.com/conan-io/conan/issues/7236
        # This only works when using conandata.yml, conanfile.py replace is broken
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class SomeBase(object):
                scm = {"type" : "git",
                       "url" : "remote1",
                       "revision" : "auto"}

            class MyConanfileBase(SomeBase, ConanFile):
                pass
            """)
        base_rev = client.init_git_repo({"conanfile.py": conanfile}, branch="my_release",
                                        folder="base")
        client.run("config set general.scm_to_conandata=1")
        client.run("export base base/1.1@user/testing")

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                name = "%s"
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.SomeBase"
            """)
        reuse1_rev = client.init_git_repo({"conanfile.py": reuse % "reuse1"}, branch="release",
                                          folder="reuse1")
        reuse2_rev = client.init_git_repo({"conanfile.py": reuse % "reuse2"}, branch="release",
                                          folder="reuse2")
        client.run("export reuse1 reuse1/1.1@user/testing")
        client.run("export reuse2 reuse2/1.1@user/testing")

        client.run("inspect base/1.1@user/testing -a=scm --json=base.json")
        base_json = json.loads(client.load("base.json"))
        client.run("inspect reuse1/1.1@user/testing -a=scm --json=reuse1.json")
        reuse1_json = json.loads(client.load("reuse1.json"))
        client.run("inspect reuse2/1.1@user/testing -a=scm --json=reuse2.json")
        reuse2_json = json.loads(client.load("reuse2.json"))
        self.assertEqual(base_json["scm"]["revision"], base_rev)
        self.assertEqual(reuse1_json["scm"]["revision"], reuse1_rev)
        self.assertEqual(reuse2_json["scm"]["revision"], reuse2_rev)
        self.assertNotEqual(base_rev, reuse1_rev)
        self.assertNotEqual(base_rev, reuse2_rev)
        self.assertNotEqual(reuse2_rev, reuse1_rev)



import json
import os
import shutil
import tarfile
import textwrap
import time
import unittest

import pytest
import six
from mock import patch
from parameterized import parameterized

from conans.client.cache.remote_registry import Remote
from conans.client.conf import ConanClientConfigParser
from conans.client.conf.config_installer import _hide_password, _ConfigOrigin
from conans.client.downloaders.file_downloader import FileDownloader
from conans.errors import ConanException
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.test_files import scan_folder, temp_folder, tgz_with_contents
from conans.test.utils.tools import TestClient, StoppableThreadBottle, zipdir
from conans.util.files import load, mkdir, save, save_files, make_file_read_only

win_profile = """[settings]
    os: Windows
"""

linux_profile = """[settings]
    os: Linux
"""

remotes = """myrepo1 https://myrepourl.net False
my-repo-2 https://myrepo2.com True
"""

registry = """myrepo1 https://myrepourl.net False

Pkg/1.0@user/channel myrepo1
"""

settings_yml = """os:
    Windows:
    Linux:
arch: [x86, x86_64]
"""

cache_conan_conf = """
[log]
run_to_output = False       # environment CONAN_LOG_RUN_TO_OUTPUT
level = 10                  # environment CONAN_LOGGING_LEVEL

[general]
compression_level = 6                 # environment CONAN_COMPRESSION_LEVEL
cpu_count = 1             # environment CONAN_CPU_COUNT
default_package_id_mode = full_package_mode # environment CONAN_DEFAULT_PACKAGE_ID_MODE

[proxies]
# Empty (or missing) section will try to use system proxies.
# As documented in https://requests.readthedocs.io/en/master/user/advanced/#proxies
http = http://user:pass@10.10.1.10:3128/
https = None
# http = http://10.10.1.10:3128
# https = http://10.10.1.10:1080
"""

myfuncpy = """def mycooladd(a, b):
    return a + b
"""

conanconf_interval = """
[general]
config_install_interval = 5m
"""


class ConfigInstallTest(unittest.TestCase):
    def setUp(self):
        self.client = TestClient()
        save(os.path.join(self.client.cache.profiles_path, "default"), "#default profile empty")
        save(os.path.join(self.client.cache.profiles_path, "linux"), "#empty linux profile")

    @staticmethod
    def _create_profile_folder(folder=None):
        folder = folder or temp_folder(path_with_spaces=False)
        save_files(folder, {"settings.yml": settings_yml,
                            "remotes.txt": remotes,
                            "profiles/linux": linux_profile,
                            "profiles/windows": win_profile,
                            "hooks/dummy": "#hook dummy",
                            "hooks/foo.py": "#hook foo",
                            "hooks/custom/custom.py": "#hook custom",
                            ".git/hooks/foo": "foo",
                            "hooks/.git/hooks/before_push": "before_push",
                            "config/conan.conf": cache_conan_conf,
                            "pylintrc": "#Custom pylint",
                            "python/myfuncs.py": myfuncpy,
                            "python/__init__.py": ""
                            })
        return folder

    def test_config_hooks(self):
        # Make sure the conan.conf hooks information is appended
        folder = temp_folder(path_with_spaces=False)
        conan_conf = textwrap.dedent("""
            [hooks]
            foo
            custom/custom
            """)
        save_files(folder, {"config/conan.conf": conan_conf})
        client = TestClient()
        client.run('config install "%s"' % folder)
        self.assertIn("Processing conan.conf", client.out)
        content = load(client.cache.conan_conf_path)
        self.assertEqual(1, content.count("foo"))
        self.assertEqual(1, content.count("custom/custom"))

        config = ConanClientConfigParser(client.cache.conan_conf_path)
        hooks = config.get_item("hooks")
        self.assertIn("foo", hooks)
        self.assertIn("custom/custom", hooks)
        self.assertIn("attribute_checker", hooks)
        client.run('config install "%s"' % folder)
        self.assertIn("Processing conan.conf", client.out)
        content = load(client.cache.conan_conf_path)
        self.assertEqual(1, content.count("foo"))
        self.assertEqual(1, content.count("custom/custom"))

    def test_config_fails_no_storage(self):
        folder = temp_folder(path_with_spaces=False)
        save_files(folder, {"remotes.txt": remotes})
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . pkg/1.0@")
        conf = load(client.cache.conan_conf_path)
        conf = conf.replace("path = ./data", "")
        save(client.cache.conan_conf_path, conf)
        client.run('config install "%s"' % folder)
        client.run("remote list")
        self.assertIn("myrepo1: https://myrepourl.net [Verify SSL: False]", client.out)
        self.assertIn("my-repo-2: https://myrepo2.com [Verify SSL: True]", client.out)

    def _create_zip(self, zippath=None):
        folder = self._create_profile_folder()
        zippath = zippath or os.path.join(folder, "myconfig.zip")
        zipdir(folder, zippath)
        return zippath

    @staticmethod
    def _get_files(folder):
        relpaths = scan_folder(folder)
        files = {}
        for path in relpaths:
            with open(os.path.join(folder, path), "r") as file_handle:
                files[path] = file_handle.read()
        return files

    def _create_tgz(self, tgz_path=None):
        folder = self._create_profile_folder()
        tgz_path = tgz_path or os.path.join(folder, "myconfig.tar.gz")
        files = self._get_files(folder)
        return tgz_with_contents(files, tgz_path)

    def _check(self, params):
        typ, uri, verify, args = [p.strip() for p in params.split(",")]
        configs = json.loads(load(self.client.cache.config_install_file))
        config = _ConfigOrigin(configs[-1])  # Check the last one
        self.assertEqual(config.type, typ)
        self.assertEqual(config.uri, uri)
        self.assertEqual(str(config.verify_ssl), verify)
        self.assertEqual(str(config.args), args)
        settings_path = self.client.cache.settings_path
        self.assertEqual(load(settings_path).splitlines(), settings_yml.splitlines())
        cache_remotes = self.client.cache.registry.load_remotes()
        self.assertEqual(list(cache_remotes.values()), [
            Remote("myrepo1", "https://myrepourl.net", False, False),
            Remote("my-repo-2", "https://myrepo2.com", True, False),
        ])
        self.assertEqual(sorted(os.listdir(self.client.cache.profiles_path)),
                         sorted(["default", "linux", "windows"]))
        self.assertEqual(load(os.path.join(self.client.cache.profiles_path, "linux")).splitlines(),
                         linux_profile.splitlines())
        self.assertEqual(load(os.path.join(self.client.cache.profiles_path,
                                           "windows")).splitlines(),
                         win_profile.splitlines())
        conan_conf = ConanClientConfigParser(self.client.cache.conan_conf_path)
        self.assertEqual(conan_conf.get_item("log.run_to_output"), "False")
        self.assertEqual(conan_conf.get_item("log.run_to_file"), "False")
        self.assertEqual(conan_conf.get_item("log.level"), "10")
        self.assertEqual(conan_conf.get_item("general.compression_level"), "6")
        self.assertEqual(conan_conf.get_item("general.default_package_id_mode"),
                         "full_package_mode")
        self.assertEqual(conan_conf.get_item("general.sysrequires_sudo"), "True")
        self.assertEqual(conan_conf.get_item("general.cpu_count"), "1")
        with six.assertRaisesRegex(self, ConanException, "'config_install' doesn't exist"):
            conan_conf.get_item("general.config_install")
        self.assertEqual(conan_conf.get_item("proxies.https"), "None")
        self.assertEqual(conan_conf.get_item("proxies.http"), "http://user:pass@10.10.1.10:3128/")
        self.assertEqual("#Custom pylint",
                         load(os.path.join(self.client.cache_folder, "pylintrc")))
        self.assertEqual("",
                         load(os.path.join(self.client.cache_folder, "python",
                                           "__init__.py")))
        self.assertEqual("#hook dummy",
                         load(os.path.join(self.client.cache_folder, "hooks", "dummy")))
        self.assertEqual("#hook foo",
                         load(os.path.join(self.client.cache_folder, "hooks", "foo.py")))
        self.assertEqual("#hook custom",
                         load(os.path.join(self.client.cache_folder, "hooks", "custom",
                                           "custom.py")))
        self.assertFalse(os.path.exists(os.path.join(self.client.cache_folder, "hooks",
                                                     ".git")))
        self.assertFalse(os.path.exists(os.path.join(self.client.cache_folder, ".git")))

    def test_reuse_python(self):
        zippath = self._create_zip()
        self.client.run('config install "%s"' % zippath)
        conanfile = """from conans import ConanFile
from myfuncs import mycooladd
a = mycooladd(1, 2)
assert a == 3
class Pkg(ConanFile):
    def build(self):
        self.output.info("A is %s" % a)
"""
        self.client.save({"conanfile.py": conanfile})
        self.client.run("create . Pkg/0.1@user/testing")
        self.assertIn("A is 3", self.client.out)

    def test_install_file(self):
        """ should install from a file in current dir
        """
        zippath = self._create_zip()
        for filetype in ["", "--type=file"]:
            self.client.run('config install "%s" %s' % (zippath, filetype))
            self._check("file, %s, True, None" % zippath)
            self.assertTrue(os.path.exists(zippath))

    def test_install_config_file(self):
        """ should install from a settings and remotes file in configuration directory
        """
        import tempfile
        profile_folder = self._create_profile_folder()
        self.assertTrue(os.path.isdir(profile_folder))
        src_setting_file = os.path.join(profile_folder, "settings.yml")
        src_remote_file = os.path.join(profile_folder, "remotes.txt")

        # Install profile_folder without settings.yml + remotes.txt in order to install them manually
        tmp_dir = tempfile.mkdtemp()
        dest_setting_file = os.path.join(tmp_dir, "settings.yml")
        dest_remote_file = os.path.join(tmp_dir, "remotes.txt")
        shutil.move(src_setting_file, dest_setting_file)
        shutil.move(src_remote_file, dest_remote_file)
        self.client.run('config install "%s"' % profile_folder)
        shutil.move(dest_setting_file, src_setting_file)
        shutil.move(dest_remote_file, src_remote_file)
        shutil.rmtree(tmp_dir)

        for cmd_option in ["", "--type=file"]:
            self.client.run('config install "%s" %s' % (src_setting_file, cmd_option))
            self.client.run('config install "%s" %s' % (src_remote_file, cmd_option))
            self._check("file, %s, True, None" % src_remote_file)

    def test_install_dir(self):
        """ should install from a dir in current dir
        """
        folder = self._create_profile_folder()
        self.assertTrue(os.path.isdir(folder))
        for dirtype in ["", "--type=dir"]:
            self.client.run('config install "%s" %s' % (folder, dirtype))
            self._check("dir, %s, True, None" % folder)

    def test_install_source_target_folders(self):
        folder = temp_folder()
        save_files(folder, {"subf/file.txt": "hello",
                            "subf/subf/file2.txt": "bye"})
        self.client.run('config install "%s" -sf=subf -tf=newsubf' % folder)
        content = load(os.path.join(self.client.cache_folder, "newsubf/file.txt"))
        self.assertEqual(content, "hello")
        content = load(os.path.join(self.client.cache_folder, "newsubf/subf/file2.txt"))
        self.assertEqual(content, "bye")

    def test_install_multiple_configs(self):
        folder = temp_folder()
        save_files(folder, {"subf/file.txt": "hello",
                            "subf2/file2.txt": "bye"})
        self.client.run('config install "%s" -sf=subf' % folder)
        content = load(os.path.join(self.client.cache_folder, "file.txt"))
        file2 = os.path.join(self.client.cache_folder, "file2.txt")
        self.assertEqual(content, "hello")
        self.assertFalse(os.path.exists(file2))
        self.client.run('config install "%s" -sf=subf2' % folder)
        content = load(file2)
        self.assertEqual(content, "bye")
        save_files(folder, {"subf/file.txt": "HELLO!!",
                            "subf2/file2.txt": "BYE!!"})
        self.client.run('config install')
        content = load(os.path.join(self.client.cache_folder, "file.txt"))
        self.assertEqual(content, "HELLO!!")
        content = load(file2)
        self.assertEqual(content, "BYE!!")

    def test_dont_duplicate_configs(self):
        folder = temp_folder()
        save_files(folder, {"subf/file.txt": "hello"})
        self.client.run('config install "%s" -sf=subf' % folder)
        self.client.run('config install "%s" -sf=subf' % folder)
        content = load(self.client.cache.config_install_file)
        self.assertEqual(1, content.count("subf"))
        self.client.run('config install "%s" -sf=other' % folder)
        content = load(self.client.cache.config_install_file)
        self.assertEqual(1, content.count("subf"))
        self.assertEqual(1, content.count("other"))

    def test_install_registry_txt_error(self):
        folder = temp_folder()
        save_files(folder, {"registry.txt": "myrepo1 https://myrepourl.net False"})
        self.client.run('config install "%s"' % folder)
        self.assertIn("WARN: registry.txt has been deprecated. Migrating to remotes.json",
                      self.client.out)
        self.client.run("remote list")
        self.assertEqual("myrepo1: https://myrepourl.net [Verify SSL: False]\n", self.client.out)

    def test_install_registry_json_error(self):
        folder = temp_folder()
        registry_json = {"remotes": [{"url": "https://server.conan.io",
                                      "verify_ssl": True,
                                      "name": "conan.io"
                                      }]}
        save_files(folder, {"registry.json": json.dumps(registry_json)})
        self.client.run('config install "%s"' % folder)
        self.assertIn("WARN: registry.json has been deprecated. Migrating to remotes.json",
                      self.client.out)
        self.client.run("remote list")
        self.assertEqual("conan.io: https://server.conan.io [Verify SSL: True]\n", self.client.out)

    def test_install_remotes_json(self):
        folder = temp_folder()

        remotes_json = textwrap.dedent("""
            {
                "remotes": [
                    { "name": "repojson1", "url": "https://repojson1.net", "verify_ssl": false },
                    { "name": "repojson2", "url": "https://repojson2.com", "verify_ssl": true }
                ]
            }
        """)

        remotes_txt = textwrap.dedent("""\
            repotxt1 https://repotxt1.net False
            repotxt2 https://repotxt2.com True
        """)

        # remotes.txt and json try to define both the remotes,
        # could lead to unpredictable results
        save_files(folder, {"remotes.json": remotes_json,
                            "remotes.txt": remotes_txt})

        self.client.run(f'config install "{folder}"')
        assert "Defining remotes from remotes.json" in self.client.out
        assert "Defining remotes from remotes.txt" in self.client.out

        # If there's only a remotes.txt it's the one installed
        folder = temp_folder()
        save_files(folder, {"remotes.txt": remotes_txt})

        self.client.run(f'config install "{folder}"')

        assert "Defining remotes from remotes.txt" in self.client.out

        self.client.run('remote list')

        assert "repotxt1: https://repotxt1.net [Verify SSL: False]" in self.client.out
        assert "repotxt2: https://repotxt2.com [Verify SSL: True]" in self.client.out

        # If there's only a remotes.json it's the one installed
        folder = temp_folder()
        save_files(folder, {"remotes.json": remotes_json})

        self.client.run(f'config install "{folder}"')
        assert "Defining remotes from remotes.json" in self.client.out

        self.client.run('remote list')

        assert "repojson1: https://repojson1.net [Verify SSL: False]" in self.client.out
        assert "repojson2: https://repojson2.com [Verify SSL: True]" in self.client.out

    def test_without_profile_folder(self):
        shutil.rmtree(self.client.cache.profiles_path)
        zippath = self._create_zip()
        self.client.run('config install "%s"' % zippath)
        self.assertEqual(sorted(os.listdir(self.client.cache.profiles_path)),
                         sorted(["linux", "windows"]))
        self.assertEqual(load(os.path.join(self.client.cache.profiles_path, "linux")).splitlines(),
                         linux_profile.splitlines())

    def test_install_url(self):
        """ should install from a URL
        """

        for origin in ["", "--type=url"]:
            def my_download(obj, url, file_path, **kwargs):  # @UnusedVariable
                self._create_zip(file_path)

            with patch.object(FileDownloader, 'download', new=my_download):
                self.client.run("config install http://myfakeurl.com/myconf.zip %s" % origin)
                self._check("url, http://myfakeurl.com/myconf.zip, True, None")

                # repeat the process to check
                self.client.run("config install http://myfakeurl.com/myconf.zip %s" % origin)
                self._check("url, http://myfakeurl.com/myconf.zip, True, None")

    def test_install_url_query(self):
        """ should install from a URL
        """

        def my_download(obj, url, file_path, **kwargs):  # @UnusedVariable
            self._create_zip(file_path)

        with patch.object(FileDownloader, 'download', new=my_download):
            # repeat the process to check it works with ?args
            self.client.run("config install http://myfakeurl.com/myconf.zip?sha=1")
            self._check("url, http://myfakeurl.com/myconf.zip?sha=1, True, None")

    def test_install_change_only_verify_ssl(self):
        def my_download(obj, url, file_path, **kwargs):  # @UnusedVariable
            self._create_zip(file_path)

        with patch.object(FileDownloader, 'download', new=my_download):
            self.client.run("config install http://myfakeurl.com/myconf.zip")
            self._check("url, http://myfakeurl.com/myconf.zip, True, None")

            # repeat the process to check
            self.client.run("config install http://myfakeurl.com/myconf.zip --verify-ssl=False")
            self._check("url, http://myfakeurl.com/myconf.zip, False, None")

    def test_install_url_tgz(self):
        """ should install from a URL to tar.gz
        """

        def my_download(obj, url, file_path, **kwargs):  # @UnusedVariable
            self._create_tgz(file_path)

        with patch.object(FileDownloader, 'download', new=my_download):
            self.client.run("config install http://myfakeurl.com/myconf.tar.gz")
            self._check("url, http://myfakeurl.com/myconf.tar.gz, True, None")

    def test_failed_install_repo(self):
        """ should install from a git repo
        """
        self.client.run('config install notexistingrepo.git', assert_error=True)
        self.assertIn("ERROR: Failed conan config install: Can't clone repo", self.client.out)

    def test_failed_install_http(self):
        """ should install from a http zip
        """
        self.client.run("config set general.retry_wait=0")
        self.client.run('config install httpnonexisting', assert_error=True)
        self.assertIn("ERROR: Failed conan config install: "
                      "Error while installing config from httpnonexisting", self.client.out)

    @pytest.mark.tool_git
    def test_install_repo(self):
        """ should install from a git repo
        """

        folder = self._create_profile_folder()
        with self.client.chdir(folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')

        self.client.run('config install "%s/.git"' % folder)
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, None" % check_path)

    @pytest.mark.tool_git
    def test_install_repo_relative(self):
        relative_folder = "./config"
        absolute_folder = os.path.join(self.client.current_folder, "config")
        mkdir(absolute_folder)
        folder = self._create_profile_folder(absolute_folder)
        with self.client.chdir(folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')

        self.client.run('config install "%s/.git"' % relative_folder)
        self._check("git, %s, True, None" % os.path.join("%s" % folder, ".git"))

    @pytest.mark.tool_git
    def test_install_custom_args(self):
        """ should install from a git repo
        """

        folder = self._create_profile_folder()
        with self.client.chdir(folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')

        self.client.run('config install "%s/.git" --args="-c init.templateDir=value"' % folder)
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, -c init.templateDir=value" % check_path)

    def test_force_git_type(self):
        client = TestClient()
        client.run('config install httpnonexisting --type=git', assert_error=True)
        self.assertIn("Can't clone repo", client.out)

    def test_force_dir_type(self):
        client = TestClient()
        client.run('config install httpnonexisting --type=dir', assert_error=True)
        self.assertIn("ERROR: Failed conan config install: No such directory: 'httpnonexisting'",
                      client.out)

    def test_force_file_type(self):
        client = TestClient()
        client.run('config install httpnonexisting --type=file', assert_error=True)
        self.assertIn("No such file or directory: 'httpnonexisting'", client.out)

    def test_force_url_type(self):
        client = TestClient()
        client.run('config install httpnonexisting --type=url', assert_error=True)
        self.assertIn("Error downloading file httpnonexisting: 'Invalid URL 'httpnonexisting'",
                      client.out)

    def test_reinstall_error(self):
        """ should use configured URL in conan.conf
        """
        self.client.run("config install", assert_error=True)
        self.assertIn("Called config install without arguments", self.client.out)

    def test_removed_credentials_from_url_unit(self):
        """
        Unit tests to remove credentials in netloc from url when using basic auth
        # https://github.com/conan-io/conan/issues/2324
        """
        url_without_credentials = r"https://server.com/resource.zip"
        url_with_credentials = r"https://test_username:test_password_123@server.com/resource.zip"
        url_hidden_password = r"https://test_username:<hidden>@server.com/resource.zip"

        # Check url is the same when not using credentials
        self.assertEqual(_hide_password(url_without_credentials), url_without_credentials)

        # Check password is hidden using url with credentials
        self.assertEqual(_hide_password(url_with_credentials), url_hidden_password)

        # Check that it works with other protocols ftp
        ftp_with_credentials = r"ftp://test_username_ftp:test_password_321@server.com/resurce.zip"
        ftp_hidden_password = r"ftp://test_username_ftp:<hidden>@server.com/resurce.zip"
        self.assertEqual(_hide_password(ftp_with_credentials), ftp_hidden_password)

        # Check function also works for file paths *unix/windows
        unix_file_path = r"/tmp/test"
        self.assertEqual(_hide_password(unix_file_path), unix_file_path)
        windows_file_path = r"c:\windows\test"
        self.assertEqual(_hide_password(windows_file_path), windows_file_path)

        # Check works with empty string
        self.assertEqual(_hide_password(''), '')

    def test_remove_credentials_config_installer(self):
        """ Functional test to check credentials are not displayed in output but are still present
        in conan configuration
        # https://github.com/conan-io/conan/issues/2324
        """
        fake_url_with_credentials = "http://test_user:test_password@myfakeurl.com/myconf.zip"
        fake_url_hidden_password = "http://test_user:<hidden>@myfakeurl.com/myconf.zip"

        def my_download(obj, url, file_path, **kwargs):  # @UnusedVariable
            self.assertEqual(url, fake_url_with_credentials)
            self._create_zip(file_path)

        with patch.object(FileDownloader, 'download', new=my_download):
            self.client.run("config install %s" % fake_url_with_credentials)

            # Check credentials are not displayed in output
            self.assertNotIn(fake_url_with_credentials, self.client.out)
            self.assertIn(fake_url_hidden_password, self.client.out)

            # Check credentials still stored in configuration
            self._check("url, %s, True, None" % fake_url_with_credentials)

    def test_ssl_verify(self):
        fake_url = "https://fakeurl.com/myconf.zip"

        def download_verify_false(obj, url, file_path, **kwargs):  # @UnusedVariable
            self.assertFalse(obj._verify_ssl)
            self._create_zip(file_path)

        def download_verify_true(obj, url, file_path, **kwargs):  # @UnusedVariable
            self.assertTrue(obj._verify_ssl)
            self._create_zip(file_path)

        with patch.object(FileDownloader, 'download', new=download_verify_false):
            self.client.run("config install %s --verify-ssl=False" % fake_url)

        with patch.object(FileDownloader, 'download', new=download_verify_true):
            self.client.run("config install %s --verify-ssl=True" % fake_url)

    @pytest.mark.tool_git
    def test_git_checkout_is_possible(self):
        folder = self._create_profile_folder()
        with self.client.chdir(folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')
            self.client.run_command('git checkout -b other_branch')
            save(os.path.join(folder, "hooks", "cust", "cust.py"), "")
            self.client.run_command('git add .')
            self.client.run_command('git commit -m "my file"')
            self.client.run_command('git tag 0.0.1')
            self.client.run_command('git checkout master')

        # Without checkout
        self.client.run('config install "%s/.git"' % folder)
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, None" % check_path)
        file_path = os.path.join(self.client.cache.hooks_path, "cust", "cust.py")
        self.assertFalse(os.path.exists(file_path))
        # With checkout tag and reuse url
        self.client.run('config install --args="-b 0.0.1"')
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, -b 0.0.1" % check_path)
        self.assertTrue(os.path.exists(file_path))
        # With checkout branch and reuse url
        self.client.run('config install --args="-b other_branch"')
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, -b other_branch" % check_path)
        self.assertTrue(os.path.exists(file_path))
        # Add changes to that branch and update
        with self.client.chdir(folder):
            self.client.run_command('git checkout other_branch')
            save(os.path.join(folder, "hooks", "other", "other.py"), "")
            self.client.run_command('git add .')
            self.client.run_command('git commit -m "my other file"')
            self.client.run_command('git checkout master')
        other_path = os.path.join(self.client.cache_folder, "hooks", "other", "other.py")
        self.assertFalse(os.path.exists(other_path))
        self.client.run('config install')
        check_path = os.path.join(folder, ".git")
        self._check("git, %s, True, -b other_branch" % check_path)
        self.assertTrue(os.path.exists(other_path))

    def test_config_install_requester(self):
        # https://github.com/conan-io/conan/issues/4169
        http_server = StoppableThreadBottle()
        path = self._create_zip()

        from bottle import static_file

        @http_server.server.get("/myconfig.zip")
        def get_zip():
            return static_file(os.path.basename(path), os.path.dirname(path))

        http_server.run_server()
        self.client.run("config install http://localhost:%s/myconfig.zip" % http_server.port)
        self.assertIn("Unzipping", self.client.out)
        http_server.stop()

    def test_error_missing_origin(self):
        path = self._create_zip()
        self.client.run('config install "%s"' % path)
        os.remove(path)
        self.client.run('config install', assert_error=True)
        self.assertIn("ERROR: Failed conan config install", self.client.out)

    def test_list_remove(self):
        path = self._create_zip()
        self.client.run('config install "%s"' % path)
        configs = json.loads(load(os.path.join(self.client.cache_folder, "config_install.json")))
        self.assertIn("myconfig.zip", configs[0]["uri"])
        self.client.run("config install --list")
        self.assertIn("myconfig.zip", self.client.out)
        self.client.run("config install --remove=0")
        configs = json.loads(load(os.path.join(self.client.cache_folder, "config_install.json")))
        self.assertEqual(0, len(configs))
        self.client.run("config install --list")
        self.assertNotIn("myconfig.zip", self.client.out)

    def test_list_empty_config(self):
        self.client.run("config install --list")
        self.assertEqual("", self.client.out)

    def test_remove_empty_config(self):
        self.client.run("config install --remove=0", assert_error=True)
        self.assertIn("There is no config data. Need to install config first.", self.client.out)

    def test_overwrite_read_only_file(self):
        source_folder = self._create_profile_folder()
        self.client.run('config install "%s"' % source_folder)
        # make existing settings.yml read-only
        make_file_read_only(self.client.cache.settings_path)
        self.assertFalse(os.access(self.client.cache.settings_path, os.W_OK))

        # config install should overwrite the existing read-only file
        self.client.run('config install "%s"' % source_folder)
        self.assertTrue(os.access(self.client.cache.settings_path, os.W_OK))

    def test_dont_copy_file_permissions(self):
        source_folder = self._create_profile_folder()
        # make source settings.yml read-only
        make_file_read_only(os.path.join(source_folder, 'remotes.txt'))

        self.client.run('config install "%s"' % source_folder)
        self.assertTrue(os.access(self.client.cache.settings_path, os.W_OK))


class ConfigInstallSchedTest(unittest.TestCase):

    def setUp(self):
        self.folder = temp_folder(path_with_spaces=False)
        save_files(self.folder, {"conan.conf": conanconf_interval})
        self.client = TestClient()
        self.client.save({"conanfile.txt": ""})

    def test_config_install_sched_file(self):
        """ Config install can be executed without restriction
        """
        self.client.run('config install "%s"' % self.folder)
        self.assertIn("Processing conan.conf", self.client.out)
        content = load(self.client.cache.conan_conf_path)
        self.assertEqual(1, content.count("config_install_interval"))
        self.assertIn("config_install_interval = 5m", content.splitlines())
        self.assertTrue(os.path.exists(self.client.cache.config_install_file))
        self.assertLess(os.path.getmtime(self.client.cache.config_install_file), time.time() + 1)

    def test_execute_more_than_once(self):
        """ Once executed by the scheduler, conan config install must executed again
            when invoked manually
        """
        self.client.run('config install "%s"' % self.folder)
        self.assertIn("Processing conan.conf", self.client.out)

        self.client.run('config install "%s"' % self.folder)
        self.assertIn("Processing conan.conf", self.client.out)
        self.assertLess(os.path.getmtime(self.client.cache.config_install_file), time.time() + 1)

    def test_sched_timeout(self):
        """ Conan config install must be executed when the scheduled time reaches
        """
        self.client.run('config install "%s"' % self.folder)
        self.client.run('config set general.config_install_interval=1m')
        self.assertNotIn("Processing conan.conf", self.client.out)
        past_time = int(time.time() - 120)  # 120 seconds in the past
        os.utime(self.client.cache.config_install_file, (past_time, past_time))

        self.client.run('search')  # any command will fire it
        self.assertIn("Processing conan.conf", self.client.out)
        self.client.run('search')  # not again, it was fired already
        self.assertNotIn("Processing conan.conf", self.client.out)
        self.client.run('config get general.config_install_interval')
        self.assertNotIn("Processing conan.conf", self.client.out)
        self.assertIn("5m", self.client.out)  # The previous 5 mins has been restored!

    def test_invalid_scheduler(self):
        """ An exception must be raised when conan_config.json is not listed
        """
        self.client.run('config install "%s"' % self.folder)
        os.remove(self.client.cache.config_install_file)
        self.client.run('config get general.config_install_interval', assert_error=True)
        self.assertIn("config_install_interval defined, but no config_install file", self.client.out)

    @parameterized.expand([("1y",), ("2015t",), ("42",)])
    def test_invalid_time_interval(self, internal):
        """ config_install_interval only accepts seconds, minutes, hours, days and weeks.
        """
        self.client.run('config set general.config_install_interval={}'.format(internal))
        # Any conan invocation will fire the configuration error
        self.client.run('install .', assert_error=True)
        self.assertIn("ERROR: Incorrect definition of general.config_install_interval: {}. "
                      "Removing it from conan.conf to avoid possible loop error.".format(internal),
                      self.client.out)
        self.client.run('install .')

    @pytest.mark.tool_git
    def test_config_install_remove_git_repo(self):
        """ config_install_interval must break when remote git has been removed
        """
        with self.client.chdir(self.folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')
        self.client.run('config install "%s/.git" --type git' % self.folder)
        self.assertIn("Processing conan.conf", self.client.out)
        self.assertIn("Repo cloned!", self.client.out)  # git clone executed by scheduled task
        folder_name = self.folder
        new_name = self.folder + "_test"
        os.rename(self.folder, new_name)
        with patch("conans.client.command.is_config_install_scheduled", return_value=True):
            self.client.run("config --help", assert_error=True)
            # scheduled task has been executed. Without a remote, the user should fix the config
            self.assertIn("ERROR: Failed conan config install: Can't clone repo", self.client.out)

            # restore the remote
            os.rename(new_name, folder_name)
            self.client.run("config --help")
            self.assertIn("Repo cloned!", self.client.out)

    @pytest.mark.tool_git
    def test_config_install_remove_config_repo(self):
        """ config_install_interval should not run when config list is empty
        """
        with self.client.chdir(self.folder):
            self.client.run_command('git init .')
            self.client.run_command('git add .')
            self.client.run_command('git config user.name myname')
            self.client.run_command('git config user.email myname@mycompany.com')
            self.client.run_command('git commit -m "mymsg"')
        self.client.run('config install "%s/.git" --type git' % self.folder)
        self.assertIn("Processing conan.conf", self.client.out)
        self.assertIn("Repo cloned!", self.client.out)
        # force scheduled time for all commands
        with patch("conans.client.conf.config_installer._is_scheduled_intervals", return_value=True):
            self.client.run("config --help")
            self.assertIn("Repo cloned!", self.client.out)  # git clone executed by scheduled task

            # config install must not run scheduled config
            self.client.run("config install --remove 0")
            self.assertEqual("", self.client.out)
            self.client.run("config install --list")
            self.assertEqual("", self.client.out)

            last_change = os.path.getmtime(self.client.cache.config_install_file)
            # without a config in configs file, scheduler only emits a warning
            self.client.run("help")
            self.assertIn("WARN: Skipping scheduled config install, "
                          "no config listed in config_install file", self.client.out)
            self.assertNotIn("Repo cloned!", self.client.out)
            # ... and updates the next schedule
            self.assertGreater(os.path.getmtime(self.client.cache.config_install_file), last_change)

    def test_config_fails_git_folder(self):
        # https://github.com/conan-io/conan/issues/8594
        folder = os.path.join(temp_folder(), ".gitlab-conan", ".conan")
        client = TestClient(cache_folder=folder)
        with client.chdir(self.folder):
            client.run_command('git init .')
            client.run_command('git add .')
            client.run_command('git config user.name myname')
            client.run_command('git config user.email myname@mycompany.com')
            client.run_command('git commit -m "mymsg"')
        assert ".gitlab-conan" in client.cache_folder
        assert os.path.basename(client.cache_folder) == ".conan"
        conf = load(client.cache.conan_conf_path)
        assert "config_install_interval = 5m" not in conf
        client.run('config install "%s/.git" --type git' % self.folder)
        conf = load(client.cache.conan_conf_path)
        assert "config_install_interval = 5m" in conf
        dirs = os.listdir(client.cache.cache_folder)
        assert ".git" not in dirs

    def test_config_install_reestructuring_source(self):
        """  https://github.com/conan-io/conan/issues/9885 """

        folder = temp_folder()
        client = TestClient()
        with client.chdir(folder):
            client.save({"profiles/debug/address-sanitizer": ""})
            client.run("config install .")

        debug_cache_folder = os.path.join(client.cache_folder, "profiles", "debug")
        assert os.path.isdir(debug_cache_folder)

        # Now reestructure the files, what it was already a directory in the cache now we want
        # it to be a file
        folder = temp_folder()
        with client.chdir(folder):
            client.save({"profiles/debug": ""})
            client.run("config install .")
        assert os.path.isfile(debug_cache_folder)

        # And now is a directory again
        folder = temp_folder()
        with client.chdir(folder):
            client.save({"profiles/debug/address-sanitizer": ""})
            client.run("config install .")
        assert os.path.isdir(debug_cache_folder)



import os
from conans.paths.package_layouts.package_cache_layout import short_path, PackageCacheLayout as PackageCacheLayout_co
from conans.model.ref import PackageReference
                                         
from conans.util.files import is_dirty
from sense2.paths import DEB_FOLDER

class PackageCacheLayout(PackageCacheLayout_co):
    
    def debs(self):
        return os.path.join(self._base_folder, DEB_FOLDER)

    @short_path
    def deb(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref, "{!r} != {!r}".format(pref.ref, self._ref)
        return os.path.join(self._base_folder, DEB_FOLDER, pref.id)
    
    @short_path
    def deb(self, pref):
        assert isinstance(pref, PackageReference)
        assert pref.ref == self._ref, "{!r} != {!r}".format(pref.ref, self._ref)
        return os.path.join(self._base_folder, DEB_FOLDER, pref.id)
    
    def deb_is_dirty(self, pref):
        deb_folder = os.path.join(self._base_folder, DEB_FOLDER, pref.id)
        return is_dirty(deb_folder)
    
    def deb_id_exists(self, deb_id):
        # The package exists if the folder exists, also for short_paths case
        deb_folder = self.deb(PackageReference(self._ref, deb_id))
        return os.path.isdir(deb_folder)


import textwrap

from conans.test.utils.tools import TestClient


def test_configure_args():
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeToolchain"
            def build(self):
                cmake = CMake(self)
                cmake.configure(variables={"MYVAR": "MYVALUE"})
                cmake.build(cli_args=["--verbosebuild"], build_tool_args=["-something"])
                cmake.test(cli_args=["--testverbose"], build_tool_args=["-testok"])

            def run(self, *args, **kwargs):
                self.output.info("MYRUN: {}".format(*args))
            """)
    client.save({"conanfile.py": conanfile})
    client.run("create . ")
    # TODO: This check is ugly, because the command line is so different in each platform,
    #  and args_to_string() is doing crazy stuff
    assert '-DMYVAR="MYVALUE"' in client.out
    assert "--verbosebuild" in client.out
    assert "-something" in client.out
    assert "--testverbose" in client.out
    assert "-testok" in client.out



from conans.model.ref import ConanFileReference


class GenConanfile(object):
    """
    USAGE:

    x = GenConanfile().with_import("import os").\
        with_setting("os").\
        with_option("shared", [True, False]).\
        with_default_option("shared", True).\
        with_build_msg("holaaa").\
        with_build_msg("adiooos").\
        with_package_file("file.txt", "hola").\
        with_package_file("file2.txt", "hola")
    """

    def __init__(self, name=None, version=None, new_import=False):
        self._imports = ["from conans import ConanFile" if not new_import
                         else "from conan import ConanFile"]
        self._name = name
        self._version = version
        self._settings = None
        self._options = None
        self._generators = None
        self._default_options = None
        self._provides = None
        self._deprecated = None
        self._package_lines = None
        self._package_files = None
        self._package_files_env = None
        self._package_files_link = None
        self._build_messages = None
        self._scm = None
        self._requires = None
        self._requirements = None
        self._build_requires = None
        self._build_requirements = None
        self._revision_mode = None
        self._package_info = None
        self._package_id_lines = None
        self._test_lines = None
        self._short_paths = None
        self._exports_sources = None
        self._exports = None
        self._cmake_build = False
        self._class_attributes = None

    def with_short_paths(self, value):
        self._short_paths = value
        return self

    def with_name(self, name):
        self._name = name
        return self

    def with_version(self, version):
        self._version = version
        return self

    def with_provides(self, provides):
        self._provides = self._provides or []
        self._provides.append(provides)
        return self

    def with_deprecated(self, deprecated):
        self._deprecated = deprecated
        return self

    def with_revision_mode(self, revision_mode):
        self._revision_mode = revision_mode
        return self

    def with_scm(self, scm):
        self._scm = scm
        return self

    def with_generator(self, generator):
        self._generators = self._generators or []
        self._generators.append(generator)
        return self

    def with_exports_sources(self, *exports):
        self._exports_sources = self._exports_sources or []
        for export in exports:
            self._exports_sources.append(export)
        return self

    def with_exports(self, *exports):
        self._exports = self._exports or []
        for export in exports:
            self._exports.append(export)
        return self

    def with_require(self, ref, private=False, override=False):
        self._requires = self._requires or []
        ref_str = ref.full_str() if isinstance(ref, ConanFileReference) else ref
        self._requires.append((ref_str, private, override))
        return self

    def with_requires(self, *refs):
        for ref in refs:
            self.with_require(ref)
        return self

    def with_requirement(self, ref, private=False, override=False):
        self._requirements = self._requirements or []
        ref_str = ref.full_str() if isinstance(ref, ConanFileReference) else ref
        self._requirements.append((ref_str, private, override))
        return self

    def with_build_requires(self, *refs):
        self._build_requires = self._build_requires or []
        for ref in refs:
            ref_str = ref.full_str() if isinstance(ref, ConanFileReference) else ref
            self._build_requires.append(ref_str)
        return self

    def with_build_requirement(self, ref, force_host_context=False):
        self._build_requirements = self._build_requirements or []
        ref_str = ref.full_str() if isinstance(ref, ConanFileReference) else ref
        self._build_requirements.append((ref_str, force_host_context))
        return self

    def with_import(self, i):
        if i not in self._imports:
            self._imports.append(i)
        return self

    def with_setting(self, setting):
        self._settings = self._settings or []
        self._settings.append(setting)
        return self

    def with_settings(self, *settings):
        self._settings = self._settings or []
        self._settings.extend(settings)
        return self

    def with_option(self, option_name, values):
        self._options = self._options or {}
        self._options[option_name] = values
        return self

    def with_default_option(self, option_name, value):
        self._default_options = self._default_options or {}
        self._default_options[option_name] = value
        return self

    def with_package_file(self, file_name, contents=None, env_var=None, link=None):
        if not contents and not env_var:
            raise Exception("Specify contents or env_var")
        self._package_files = self._package_files or {}
        self._package_files_link = self._package_files_link or {}
        self._package_files_env = self._package_files_env or {}
        self.with_import("import os")
        self.with_import("from conans import tools")
        if contents:
            self._package_files[file_name] = contents
        if link:
            self._package_files_link[file_name] = link
        if env_var:
            self._package_files_env[file_name] = env_var
        return self

    def with_package(self, *lines):
        self._package_lines = self._package_lines or []
        for line in lines:
            self._package_lines.append(line)
        return self

    def with_build_msg(self, msg):
        self._build_messages = self._build_messages or []
        self._build_messages.append(msg)
        return self

    def with_package_info(self, cpp_info=None, env_info=None):
        assert isinstance(cpp_info, dict), "cpp_info ({}) expects dict".format(type(cpp_info))
        assert isinstance(env_info, dict), "env_info ({}) expects dict".format(type(env_info))
        self._package_info = self._package_info or {}
        if cpp_info:
            self._package_info["cpp_info"] = cpp_info
        if env_info:
            self._package_info["env_info"] = env_info
        return self

    def with_package_id(self, line):
        self._package_id_lines = self._package_id_lines or []
        self._package_id_lines.append(line)
        return self

    def with_test(self, line):
        self._test_lines = self._test_lines or []
        self._test_lines.append(line)
        return self

    def with_cmake_build(self):
        self._imports.append("from conan.tools.cmake import CMake")
        self._generators = self._generators or []
        self._generators.append("CMakeDeps")
        self._generators.append("CMakeToolchain")
        self.with_settings("os", "compiler", "arch", "build_type")
        self._cmake_build = True
        return self

    def with_class_attribute(self, attr):
        """.with_class_attribute("no_copy_sources=True") """
        self._class_attributes = self._class_attributes or []
        self._class_attributes.append(attr)
        return self

    @property
    def _name_render(self):
        return "name = '{}'".format(self._name)

    @property
    def _version_render(self):
        return "version = '{}'".format(self._version)

    @property
    def _provides_render(self):
        line = ", ".join('"{}"'.format(provide) for provide in self._provides)
        return "provides = {}".format(line)

    @property
    def _deprecated_render(self):
        return "deprecated = {}".format(self._deprecated)

    @property
    def _scm_render(self):
        line = ", ".join('"%s": "%s"' % (k, v) for k, v in self._scm.items())
        return "scm = {%s}" % line

    @property
    def _generators_render(self):
        line = ", ".join('"{}"'.format(generator) for generator in self._generators)
        return "generators = {}".format(line)

    @property
    def _revision_mode_render(self):
        line = "revision_mode=\"{}\"".format(self._revision_mode)
        return line

    @property
    def _settings_render(self):
        line = ", ".join('"%s"' % s for s in self._settings)
        return "settings = {}".format(line)

    @property
    def _options_render(self):
        line = ", ".join('"%s": %s' % (k, v) for k, v in self._options.items())
        tmp = "options = {%s}" % line
        return tmp

    @property
    def _default_options_render(self):
        line = ", ".join('"%s": %s' % (k, v) for k, v in self._default_options.items())
        tmp = "default_options = {%s}" % line
        return tmp

    @property
    def _build_requirements_render(self):
        lines = []
        for ref, force_host_context in self._build_requirements:
            force_host = ", force_host_context=True" if force_host_context else ""
            lines.append('        self.build_requires("{}"{})'.format(ref, force_host))
        return "def build_requirements(self):\n{}\n".format("\n".join(lines))

    @property
    def _build_requires_render(self):
        line = ", ".join(['"{}"'.format(r) for r in self._build_requires])
        tmp = "build_requires = %s" % line
        return tmp

    @property
    def _requires_render(self):
        items = []
        for ref, private, override in self._requires:
            if private or override:
                private_str = ", 'private'" if private else ""
                override_str = ", 'override'" if override else ""
                items.append('("{}"{}{})'.format(ref, private_str, override_str))
            else:
                items.append('"{}"'.format(ref))
        tmp = "requires = ({}, )".format(", ".join(items))
        return tmp

    @property
    def _requirements_render(self):
        lines = []
        for ref, private, override in self._requirements:
            private_str = ", private=True" if private else ""
            override_str = ", override=True" if override else ""
            lines.append('        self.requires("{}"{}{})'.format(ref, private_str, override_str))

        return """
    def requirements(self):
{}
        """.format("\n".join(lines))

    @property
    def _package_method(self):
        return (self._package_lines or self._package_files or self._package_files_env or
                self._package_files_link)

    @property
    def _package_method_render(self):
        lines = []
        if self._package_lines:
            lines.extend("        {}".format(line) for line in self._package_lines)
        if self._package_files:
            lines = ['        tools.save(os.path.join(self.package_folder, "{}"), "{}")'
                     ''.format(key, value)
                     for key, value in self._package_files.items()]

        if self._package_files_env:
            lines.extend(['        tools.save(os.path.join(self.package_folder, "{}"), '
                          'os.getenv("{}"))'.format(key, value)
                          for key, value in self._package_files_env.items()])
        if self._package_files_link:
            lines.extend(['        with tools.chdir(os.path.dirname('
                          'os.path.join(self.package_folder, "{}"))):\n'
                          '            os.symlink(os.path.basename("{}"), '
                          'os.path.join(self.package_folder, "{}"))'.format(key, key, value)
                          for key, value in self._package_files_link.items()])

        if not lines:
            return ""
        return """
    def package(self):
{}
    """.format("\n".join(lines))

    @property
    def _build_render(self):
        if not self._build_messages and not self._cmake_build:
            return None
        lines = []
        if self._build_messages:
            lines = ['        self.output.warn("{}")'.format(m) for m in self._build_messages]
        if self._cmake_build:
            lines.extend(['        cmake = CMake(self)',
                          '        cmake.configure()',
                          '        cmake.build()'])
        return """
    def build(self):
{}
    """.format("\n".join(lines))

    @property
    def _package_info_render(self):
        lines = []
        if "cpp_info" in self._package_info:
            for k, v in self._package_info["cpp_info"].items():
                if k == "components":
                    for comp_name, comp in v.items():
                        for comp_attr_name, comp_attr_value in comp.items():
                            lines.append('        self.cpp_info.components["{}"].{} = {}'.format(
                                comp_name, comp_attr_name, str(comp_attr_value)))
                else:
                    lines.append('        self.cpp_info.{} = {}'.format(k, str(v)))
        if "env_info" in self._package_info:
            for k, v in self._package_info["env_info"].items():
                lines.append('        self.env_info.{} = {}'.format(k, str(v)))

        return """
    def package_info(self):
{}
        """.format("\n".join(lines))

    @property
    def _package_id_lines_render(self):
        lines = ['        {}'.format(line) for line in self._package_id_lines]
        return """
    def package_id(self):
{}
        """.format("\n".join(lines))

    @property
    def _test_lines_render(self):
        if not self._test_lines:
            return ""
        lines = ['', '    def test(self):'] + ['        %s' % m for m in self._test_lines]
        return "\n".join(lines)

    @property
    def _short_paths_render(self):
        return "short_paths = {}".format(str(self._short_paths))

    @property
    def _exports_sources_render(self):
        line = ", ".join('"{}"'.format(e) for e in self._exports_sources)
        return "exports_sources = {}".format(line)

    @property
    def _exports_render(self):
        line = ", ".join('"{}"'.format(e) for e in self._exports)
        return "exports = {}".format(line)

    @property
    def _class_attributes_render(self):
        self._class_attributes = self._class_attributes or []
        return ["    {}".format(a) for a in self._class_attributes]

    def __repr__(self):
        ret = []
        ret.extend(self._imports)
        ret.append("class HelloConan(ConanFile):")

        # FIXME: This is all a mess. Replace with Jinja2
        for member in ("name", "version", "provides", "deprecated", "short_paths", "exports_sources",
                       "exports", "generators", "requires", "build_requires", "requirements",
                       "build_requirements", "scm", "revision_mode", "settings", "options",
                       "default_options", "package_method", "package_info",
                       "package_id_lines", "test_lines"
                       ):
            v = getattr(self, "_{}".format(member), None)
            if v is not None:
                ret.append("    {}".format(getattr(self, "_{}_render".format(member))))

        ret.extend(self._class_attributes_render)
        build = self._build_render
        if build is not None:
            ret.append("    {}".format(self._build_render))
        if ret[-1] == "class HelloConan(ConanFile):":
            ret.append("    pass")
        return "\n".join(ret)



import os
import platform
import re
import textwrap

import pytest
from jinja2 import Template

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient

"""
Check that the link order of libraries is preserved when using CMake generators
    https://github.com/conan-io/conan/issues/6280

"""


conanfile = Template(textwrap.dedent("""
    from conans import ConanFile

    class Recipe(ConanFile):
        name = "{{ref.name}}"
        version = "{{ref.version}}"

        {% if requires %}
        requires =
            {%- for req in requires -%}
            "{{ req }}"{% if not loop.last %}, {% endif %}
            {%- endfor -%}
        {% endif %}

        def build(self):
            with open("lib" + self.name + ".a", "w+") as f:
                f.write("fake library content")
            with open(self.name + ".lib", "w+") as f:
                f.write("fake library content")

            {% for it in libs_extra %}
            with open("lib{{ it }}.a", "w+") as f:
                f.write("fake library content")
            with open("{{ it }}.lib", "w+") as f:
                f.write("fake library content")
            {% endfor %}

        def package(self):
            self.copy("*.a", dst="lib")
            self.copy("*.lib", dst="lib")

        def package_info(self):
            self.cpp_info.includedirs = []
            # Libraries
            self.cpp_info.libs = ["{{ ref.name }}"]
            {% for it in libs_extra %}
            self.cpp_info.libs.append("{{ it }}")
            {% endfor %}
            {% for it in libs_system %}
            self.cpp_info.libs.append("{{ it }}")
            {% endfor %}

            {% if system_libs %}
            self.cpp_info.system_libs = [
                {%- for it in system_libs -%}
                "{{ it }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}]
            {% endif %}

            {% if frameworks %}
            self.cpp_info.frameworks.extend([
                {%- for it in frameworks -%}
                "{{ it }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}])
            {% endif %}
"""))

conanfile_headeronly = Template(textwrap.dedent("""
    from conans import ConanFile

    class HeaderOnly(ConanFile):
        name = "{{ref.name}}"
        version = "{{ref.version}}"

        {% if requires %}
        requires =
            {%- for req in requires -%}
            "{{ req }}"{% if not loop.last %}, {% endif %}
            {%- endfor -%}
        {% endif %}

        def package_id(self):
            self.info.header_only()

        def package_info(self):
            self.cpp_info.includedirs = []
            # It may declare system libraries
            {% for it in libs_system %}
            self.cpp_info.libs.append("{{ it }}")
            {% endfor %}

            {% if system_libs %}
            self.cpp_info.system_libs = [
                {%- for it in system_libs -%}
                "{{ it }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}]
            {% endif %}

            {% if frameworks %}
            self.cpp_info.frameworks.extend([
                {%- for it in frameworks -%}
                "{{ it }}"{% if not loop.last %}, {% endif %}
                {%- endfor -%}])
            {% endif %}
"""))

main_cpp = textwrap.dedent("""
    int main() {return 0;}
""")


@pytest.fixture(scope="module")
def client():
    libZ_ref = ConanFileReference.loads("libZ/version")
    libH2_ref = ConanFileReference.loads("header2/version")
    libH_ref = ConanFileReference.loads("header/version")
    libA_ref = ConanFileReference.loads("libA/version")
    libB_ref = ConanFileReference.loads("libB/version")
    libC_ref = ConanFileReference.loads("libC/version")
    libD_ref = ConanFileReference.loads("libD/version")

    t = TestClient(path_with_spaces=False)
    t.save({
        'libZ/conanfile.py': conanfile.render(
            ref=libZ_ref,
            libs_extra=["Z2"],
            system_libs=["system_lib"],
            frameworks=["Carbon"]),
        'libH2/conanfile.py': conanfile_headeronly.render(
            ref=libH2_ref,
            system_libs=["header2_system_lib"],
            frameworks=["Security"]),
        'libH/conanfile.py': conanfile_headeronly.render(
            ref=libH_ref,
            requires=[libH2_ref, libZ_ref],
            system_libs=["header_system_lib"],
            frameworks=["CoreAudio"]),
        'libA/conanfile.py': conanfile.render(
            ref=libA_ref,
            requires=[libH_ref],
            libs_extra=["A2"],
            system_libs=["system_lib"],
            frameworks=["Carbon"]),
        'libB/conanfile.py': conanfile.render(
            ref=libB_ref,
            requires=[libA_ref],
            libs_extra=["B2"],
            system_libs=["system_lib"],
            frameworks=["Carbon"]),
        'libC/conanfile.py': conanfile.render(
            ref=libC_ref,
            requires=[libA_ref],
            libs_extra=["C2"],
            system_libs=["system_lib"],
            frameworks=["Carbon"]),
        'libD/conanfile.py': conanfile.render(
            ref=libD_ref,
            requires=[libB_ref, libC_ref],
            libs_extra=["D2"],
            system_libs=["system_lib"],
            frameworks=["Carbon"]),
    })

    # Create all of them
    t.run("create libZ")
    t.run("create libH2")
    t.run("create libH")
    t.run("create libA")
    t.run("create libB")
    t.run("create libC")
    t.run("create libD")
    return t


def _validate_link_order(libs):
    # Check that all the libraries are there:
    assert len(libs) == 16 if platform.system() == "Darwin" else (13 if platform.system() == "Linux"
                                                                  else 23)
    # - Regular libs
    ext = ".lib" if platform.system() == "Windows" else ".a"
    prefix = "" if platform.system() == "Windows" else "lib"
    expected_libs = {prefix + it + ext for it in ['libD', 'D2', 'libB', 'B2', 'libC', 'C2',
                                                  'libA', 'A2', 'libZ', 'Z2']}
    # - System libs
    ext_system = ".lib" if platform.system() == "Windows" else ""
    expected_libs.update([it + ext_system for it in ['header_system_lib',
                                                     'header2_system_lib',
                                                     'system_lib']])
    # - Add MacOS frameworks
    if platform.system() == "Darwin":
        expected_libs.update(['CoreAudio', 'Security', 'Carbon'])
    # - Add Windows libs
    if platform.system() == "Windows":
        expected_libs.update(['kernel32.lib', 'user32.lib', 'gdi32.lib', 'winspool.lib',
                              'shell32.lib', 'ole32.lib', 'oleaut32.lib', 'uuid.lib',
                              'comdlg32.lib', 'advapi32.lib'])
    assert set(libs) == expected_libs

    # These are the first libraries and order is mandatory
    mandatory_1 = [prefix + it + ext for it in ['libD', 'D2', 'libB', 'B2', 'libC',
                                                'C2', 'libA', 'A2', ]]
    assert mandatory_1 == libs[:len(mandatory_1)]

    # Then, libZ ones must be before system libraries that are consuming
    assert libs.index(prefix + 'libZ' + ext) < libs.index('system_lib' + ext_system)
    assert libs.index(prefix + 'Z2' + ext) < libs.index('system_lib' + ext_system)

    if platform.system() == "Darwin":
       assert libs.index('liblibZ.a') < libs.index('Carbon')
       assert libs.index('libZ2.a') < libs.index('Carbon')


def _get_link_order_from_cmake(content):
    libs = []
    for it in content.splitlines():
        # This is for Linux and Mac
        # Remove double spaces from output that appear in some platforms
        line = ' '.join(it.split())
        if 'main.cpp.o -o example' in line:
            _, links = line.split("main.cpp.o -o example")
            for it_lib in links.split():
                if it_lib.startswith("-L") or it_lib.startswith("-Wl,-rpath"):
                    continue
                elif it_lib.startswith("-l"):
                    libs.append(it_lib[2:])
                elif it_lib == "-framework":
                    continue
                else:
                    try:
                        _, libname = it_lib.rsplit('/', 1)
                    except ValueError:
                        libname = it_lib
                    finally:
                        libs.append(libname)
            break
        # Windows
        if 'example.exe" /INCREMENTAL:NO /NOLOGO' in it:
            for it_lib in it.split():
                it_lib = it_lib.strip()
                if it_lib.endswith(".lib"):
                    try:
                        _, libname = it_lib.rsplit('\\', 1)
                    except ValueError:
                        libname = it_lib
                    finally:
                        libs.append(libname)
            break
    return libs


def _get_link_order_from_xcode(content):
    libs = []

    # Find the right Release block in the XCode file
    results = re.finditer('/\* Release \*/ = {', content)
    for r in results:
        release_section = content[r.start():].split("name = Release;", 1)[0]
        if "-headerpad_max_install_names" in release_section:
            break
    else:
        raise Exception("Cannot find the Release block linking the expected libraries")

    start_key = '-Wl,-headerpad_max_install_names'
    end_key = ');'
    libs_content = release_section.split(start_key, 1)[1].split(end_key, 1)[0]
    libs_unstripped = libs_content.split(",")
    for lib in libs_unstripped:
        if ".a" in lib:
            libs.append(lib.strip('"').rsplit('/', 1)[1])
        elif "-l" in lib:
            libs.append(lib.strip('"')[2:])
        elif "-framework" in lib:
            libs.append(lib.strip('"')[11:])
    return libs


def _create_find_package_project(client):
    t = TestClient(cache_folder=client.cache_folder)
    t.save({
        'conanfile.txt': textwrap.dedent("""
            [requires]
            libD/version
            [generators]
            CMakeDeps
            CMakeToolchain
            """),
        'CMakeLists.txt': textwrap.dedent("""
            cmake_minimum_required(VERSION 3.15)
            project(executable CXX)

            find_package(libD)
            add_executable(example main.cpp)
            target_link_libraries(example libD::libD)
            """),
        'main.cpp': main_cpp
    })

    t.run("install . -s build_type=Release")
    return t


def _run_and_get_lib_order(t, generator):
    if generator == "Xcode":
        t.run_command("cmake . -G Xcode -DCMAKE_VERBOSE_MAKEFILE:BOOL=True"
                      " -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake")
        # This is building by default the Debug configuration that contains nothing, so it works
        t.run_command("cmake --build .")
        # This is building the release and fails because invented system libraries are missing
        t.run_command("cmake --build . --config Release", assert_error=True)
        # Get the actual link order from the CMake call
        libs = _get_link_order_from_xcode(t.load(os.path.join('executable.xcodeproj',
                                                              'project.pbxproj')))
    else:
        t.run_command("cmake . -DCMAKE_VERBOSE_MAKEFILE:BOOL=True"
                      " -DCMAKE_BUILD_TYPE=Release"
                      " -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake")
        extra_build = "--config Release" if platform.system() == "Windows" else ""  # Windows VS
        t.run_command("cmake --build . {}".format(extra_build), assert_error=True)
        # Get the actual link order from the CMake call
        libs = _get_link_order_from_cmake(str(t.out))
    return libs


@pytest.mark.parametrize("generator", [None, "Xcode"])
@pytest.mark.tool_cmake(version="3.19")
def test_cmake_deps(client, generator):
    if generator == "Xcode" and platform.system() != "Darwin":
        pytest.skip("Xcode is needed")

    t = _create_find_package_project(client)
    libs = _run_and_get_lib_order(t, generator)
    _validate_link_order(libs)



import textwrap
import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class PythonRequiresPackageIDTest(unittest.TestCase):

    def setUp(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . tool/1.1.1@")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires ="tool/[*]"
            """)
        client2 = TestClient(cache_folder=client.cache_folder)
        client2.save({"conanfile.py": conanfile})
        self.client = client
        self.client2 = client2

    def test_default(self):
        self.client2.run("create . pkg/0.1@")
        self.assertEqual(1, str(self.client2.out).count(" resolved to 'tool/1.1.1' in local cache"))
        self.assertIn("tool/1.1.1", self.client2.out)
        self.assertIn("pkg/0.1:ecc024bdf63d1355af81d60281c569492d98901c - Build", self.client2.out)

        self.client.run("export . tool/1.1.2@")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.2", self.client2.out)
        self.assertIn("pkg/0.1:ecc024bdf63d1355af81d60281c569492d98901c - Build", self.client2.out)

        # With a minor change, it fires a rebuild
        self.client.run("export . tool/1.2.0@")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.2.0", self.client2.out)
        self.assertIn("pkg/0.1:2f471d7bec8ea131369a4b683cd1aeea65a18861 - Build", self.client2.out)

    def test_change_mode_conf(self):
        # change the policy in conan.conf
        self.client2.run("config set general.default_python_requires_id_mode=patch_mode")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.1", self.client2.out)
        self.assertIn("pkg/0.1:f3161fafc8273fe3c8afa3b51dcc198c33f66033 - Build", self.client2.out)

        # with a patch change, new ID
        self.client.run("export . tool/1.1.2@")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.2", self.client2.out)
        self.assertIn("pkg/0.1:387c1c797a011d426ecb25a1e01b28251e443ec8 - Build", self.client2.out)

    def test_unrelated_conf(self):
        # change the policy in conan.conf
        self.client2.run("config set general.default_python_requires_id_mode=unrelated_mode")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.1", self.client2.out)
        self.assertIn("pkg/0.1:c941ae50e2daf4a118c393591cfef6a55cd1cfad - Build", self.client2.out)

        # with any change the package id doesn't change
        self.client.run("export . tool/1.1.2@")
        self.client2.run("create . pkg/0.1@ --build missing")
        self.assertIn("tool/1.1.2", self.client2.out)
        self.assertIn("pkg/0.1:c941ae50e2daf4a118c393591cfef6a55cd1cfad - Cache", self.client2.out)

    def test_change_mode_package_id(self):
        # change the policy in package_id
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires ="tool/[*]"
                def package_id(self):
                    self.info.python_requires.patch_mode()
            """)
        self.client2.save({"conanfile.py": conanfile})
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.1", self.client2.out)
        self.assertIn("pkg/0.1:f3161fafc8273fe3c8afa3b51dcc198c33f66033 - Build", self.client2.out)

        # with a patch change, new ID
        self.client.run("export . tool/1.1.2@")
        self.client2.run("create . pkg/0.1@")
        self.assertIn("tool/1.1.2", self.client2.out)
        self.assertIn("pkg/0.1:387c1c797a011d426ecb25a1e01b28251e443ec8 - Build", self.client2.out)


class PythonRequiresForBuildRequiresPackageIDTest(unittest.TestCase):

    def test(self):
        client = TestClient()
        client.run("config set general.default_python_requires_id_mode=full_version_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . tool/1.1.1@")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires ="tool/[>=0.0]"
            """)

        client2 = TestClient(cache_folder=client.cache_folder)
        client2.save({"conanfile.py": conanfile,
                     "myprofile": "[build_requires]\ntool/[>=0.0]\n"})

        client2.run("create . pkg/0.1@ -pr=myprofile")
        self.assertIn("tool/1.1.1", client2.out)
        self.assertIn("pkg/0.1: Package 'f3161fafc8273fe3c8afa3b51dcc198c33f66033' created",
                      client2.out)

        client.run("create . tool/1.1.2@")
        client2.run("install pkg/0.1@ -pr=myprofile", assert_error=True)
        self.assertIn("ERROR: Missing binary: pkg/0.1:387c1c797a011d426ecb25a1e01b28251e443ec8",
                      client2.out)
        self.assertIn("tool/1.1.2", client2.out)
        self.assertNotIn("tool/1.1.1", client2.out)

        client2.run("create . pkg/0.1@ -pr=myprofile")
        self.assertIn("pkg/0.1: Applying build-requirement: tool/1.1.2", client2.out)
        self.assertIn("pkg/0.1: Package '387c1c797a011d426ecb25a1e01b28251e443ec8' created",
                      client2.out)



import textwrap

from conans.test.utils.tools import TestClient


def test_custom_block():
    # https://github.com/conan-io/conan/issues/9998
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMakeToolchain
        class Pkg(ConanFile):
            def generate(self):
                toolchain = CMakeToolchain(self)

                class MyBlock:
                    template = "Hello {{myvar}}!!!"

                    def context(self):
                        return {"myvar": "World"}

                toolchain.blocks["mynewblock"] = MyBlock
                toolchain.generate()
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install .")
    assert "Hello World!!!" in c.load("conan_toolchain.cmake")



import os
import platform
import textwrap
import unittest

import pytest

from parameterized import parameterized
from conans.client.tools.env import environment_append
from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_cmake
class CMakeAppleFrameworksTestCase(unittest.TestCase):
    lib_ref = ConanFileReference.loads("lib/version")
    lib_conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Lib(ConanFile):
            def package_info(self):
                self.cpp_info.frameworks.extend(['Foundation', 'CoreServices', 'CoreFoundation'])
    """)

    app_conanfile = textwrap.dedent("""
        from conans import ConanFile, CMake

        class App(ConanFile):
            requires = "{}"
            generators = "{{generator}}"
            settings = "build_type",  # cmake_multi doesn't work without build_type

            def build(self):
                cmake = CMake(self)
                cmake.configure()
    """.format(lib_ref))

    def setUp(self):
        self.t = TestClient()
        self.t.save({'conanfile.py': self.lib_conanfile})
        self.t.run("create . {}@".format(self.lib_ref))

    def _check_frameworks_found(self, output):
        self.assertIn("/System/Library/Frameworks/Foundation.framework;", output)
        self.assertIn("/System/Library/Frameworks/CoreServices.framework;", output)
        self.assertIn("/System/Library/Frameworks/CoreFoundation.framework", output)

    def test_apple_framework_cmake(self):
        app_cmakelists = textwrap.dedent("""
            project(Testing CXX)

            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup()

            message(">>> CONAN_FRAMEWORKS_FOUND_LIB: ${CONAN_FRAMEWORKS_FOUND_LIB}")
        """)

        self.t.save({'conanfile.py': self.app_conanfile.format(generator="cmake"),
                     'CMakeLists.txt': app_cmakelists})
        self.t.run("install .")
        self.t.run("build .")
        self._check_frameworks_found(str(self.t.out))

    def test_apple_framework_cmake_multi(self):
        app_cmakelists = textwrap.dedent("""
            project(Testing CXX)

            include(${CMAKE_BINARY_DIR}/conanbuildinfo_multi.cmake)
            conan_basic_setup()

            message(">>> CONAN_FRAMEWORKS_FOUND_LIB_DEBUG: ${CONAN_FRAMEWORKS_FOUND_LIB_DEBUG}")
            message(">>> CONAN_FRAMEWORKS_FOUND_LIB_RELEASE: ${CONAN_FRAMEWORKS_FOUND_LIB_RELEASE}")
        """)

        self.t.save({'conanfile.py': self.app_conanfile.format(generator="cmake_multi"),
                     'CMakeLists.txt': app_cmakelists})
        self.t.run("install . -s build_type=Release")
        self.t.run("install . -s build_type=Debug")
        self.t.run("build .")
        self._check_frameworks_found(str(self.t.out))

    @pytest.mark.tool_cmake(version="3.19")
    def test_apple_framework_cmake_multi_xcode(self):
        app_cmakelists = textwrap.dedent("""
            project(Testing CXX)

            include(${CMAKE_BINARY_DIR}/conanbuildinfo_multi.cmake)
            conan_basic_setup()

            message(">>> CONAN_FRAMEWORKS_FOUND_LIB_DEBUG: ${CONAN_FRAMEWORKS_FOUND_LIB_DEBUG}")
            message(">>> CONAN_FRAMEWORKS_FOUND_LIB_RELEASE: ${CONAN_FRAMEWORKS_FOUND_LIB_RELEASE}")
        """)

        self.t.save({'conanfile.py': self.app_conanfile.format(generator="cmake_multi"),
                     'CMakeLists.txt': app_cmakelists})
        with environment_append({"CONAN_CMAKE_GENERATOR": "Xcode"}):
            self.t.run("install . -s build_type=Release")
            self.t.run("install . -s build_type=Debug")
            self.t.run("build .")
            self._check_frameworks_found(str(self.t.out))

    def test_apple_framework_cmake_find_package(self):
        app_cmakelists = textwrap.dedent("""
            project(Testing CXX)

            find_package(lib)

            message(">>> CONAN_FRAMEWORKS_FOUND_LIB: ${lib_FRAMEWORKS_FOUND}")
        """)

        self.t.save({'conanfile.py': self.app_conanfile.format(generator="cmake_find_package"),
                     'CMakeLists.txt': app_cmakelists})
        self.t.run("install .")
        self.t.run("build .")
        self._check_frameworks_found(str(self.t.out))


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
class CMakeAppleOwnFrameworksTestCase(unittest.TestCase):
    conanfile = textwrap.dedent("""
                from conans import ConanFile, CMake, tools

                class AppleframeworkConan(ConanFile):
                    settings = "os", "compiler", "build_type", "arch"
                    generators = "cmake"
                    exports_sources = "src/*"
                    name = "mylibrary"
                    version = "1.0"
                    def build(self):
                        cmake = CMake(self)
                        xcrun = tools.XCRun(self.settings)
                        cmake.definitions.update({
                            'CMAKE_OSX_SYSROOT' : xcrun.sdk_path,
                            'CMAKE_OSX_ARCHITECTURES' : tools.to_apple_arch(self.settings.arch),
                        })
                        cmake.configure(source_folder="src")
                        cmake.build()
                        cmake.install()
                        self.run("otool -L '%s/lib/hello.framework/hello'" % self.build_folder)
                        self.run("otool -L '%s/hello.framework/hello'" % self.package_folder)

                    def package_info(self):
                        self.cpp_info.frameworkdirs.append(self.package_folder)
                        self.cpp_info.frameworks.append("hello")
                """)
    cmake = textwrap.dedent("""
                cmake_minimum_required(VERSION 2.8)
                project(MyHello CXX)

                include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
                conan_basic_setup()

                # set @rpaths for libraries to link against
                SET(CMAKE_SKIP_RPATH FALSE)
                #SET(CMAKE_SKIP_BUILD_RPATH  FALSE)
                #SET(CMAKE_INSTALL_RPATH "@rpath/")
                #SET(CMAKE_INSTALL_RPATH_USE_LINK_PATH TRUE)

                add_library(hello SHARED hello.cpp hello.h)
                set_target_properties(hello PROPERTIES
                  FRAMEWORK TRUE
                  FRAMEWORK_VERSION A
                  MACOSX_FRAMEWORK_IDENTIFIER com.cmake.hello
                  MACOSX_FRAMEWORK_INFO_PLIST src/Info.plist
                  # "current version" in semantic format in Mach-O binary file
                  VERSION 1.6.0
                  # "compatibility version" in semantic format in Mach-O binary file
                  SOVERSION 1.6.0
                  PUBLIC_HEADER hello.h
                  INSTALL_NAME_DIR "@rpath"
                  MACOSX_RPATH TRUE
                )
                install(TARGETS hello DESTINATION ".")
            """)
    hello_h = textwrap.dedent("""
                #pragma once

                #ifdef WIN32
                  #define HELLO_EXPORT __declspec(dllexport)
                #else
                  #define HELLO_EXPORT __attribute__((visibility("default")))
                #endif

                #ifdef __cplusplus
                extern "C" {
                #endif
                class HELLO_EXPORT Hello
                {
                    public:
                        static void hello();
                };
                #ifdef __cplusplus
                }
                #endif
            """)
    hello_cpp = textwrap.dedent("""
                #include <iostream>
                #include "hello.h"

                void Hello::hello(){
                    #ifdef NDEBUG
                    std::cout << "Hello World Release!" <<std::endl;
                    #else
                    std::cout << "Hello World Debug!" <<std::endl;
                    #endif
                }
            """)
    infoplist = textwrap.dedent("""
                <?xml version="1.0" encoding="UTF-8"?>
                <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN"
                         "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
                <plist version="1.0">
                <dict>
                    <key>CFBundleDisplayName</key>
                    <string>hello</string>
                    <key>CFBundleExecutable</key>
                    <string>hello</string>
                    <key>CFBundleIdentifier</key>
                    <string>com.test.hello</string>
                    <key>CFBundleInfoDictionaryVersion</key>
                    <string>6.0</string>
                    <key>CFBundleName</key>
                    <string>hello</string>
                    <key>CFBundlePackageType</key>
                    <string>FMWK</string>
                    <key>CFBundleShortVersionString</key>
                    <string>1.6.0</string>
                    <key>CFBundleVersion</key>
                    <string>1.6.0</string>
                    <key>Flavor_ID</key>
                    <string>0</string>
                    <key>NSAppTransportSecurity</key>
                    <dict>
                        <key>NSAllowsArbitraryLoads</key>
                        <true/>
                    </dict>
                    <key>NSPrincipalClass</key>
                    <string></string>
                </dict>
                </plist>
            """)
    timer_cpp = textwrap.dedent("""
        #include <hello/hello.h>
        int main(){
            Hello::hello();
        }
        """)

    @parameterized.expand([('',),
                           ('-s os=iOS -s os.version=10.0 -s arch=armv8',),
                           ("-s os=tvOS -s os.version=11.0 -s arch=armv8",)])
    def test_apple_own_framework_cmake(self, settings):
        client = TestClient()

        test_cmake = textwrap.dedent("""
            project(Testing CXX)
            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup()
            message(">>> CONAN_FRAMEWORKS_FOUND_MYLIBRARY: ${CONAN_FRAMEWORKS_FOUND_MYLIBRARY}")
            add_executable(timer timer.cpp)
            target_link_libraries(timer ${CONAN_LIBS})
        """)

        test_conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake, tools
            class TestPkg(ConanFile):
                generators = "cmake"
                settings = "os", "arch", "compiler", "build_type"
                def build(self):
                    cmake = CMake(self)
                    cmake_system_name = {"Macos" : "Darwin",
                                         "iOS" : "iOS",
                                         "tvOS" : "tvOS"}[str(self.settings.os)]
                    archs = {
                        "Macos": "x86_64",
                        "iOS": "arm64;x86_64",
                        "tvOS": "arm64;x86_64",
                        }[str(self.settings.os)]
                    xcrun = tools.XCRun(self.settings)
                    cmake.definitions.update({
                        'CMAKE_OSX_SYSROOT': xcrun.sdk_path,
                        'CMAKE_SYSTEM_NAME': cmake_system_name,
                    })
                    cmake.configure()
                    cmake.build()
                def test(self):
                    if not tools.cross_building(self):
                        self.run("bin/timer", run_environment=True)
            """)
        client.save({'conanfile.py': self.conanfile,
                     "src/CMakeLists.txt": self.cmake,
                     "src/hello.h": self.hello_h,
                     "src/hello.cpp": self.hello_cpp,
                     "src/Info.plist": self.infoplist,
                     "test_package/conanfile.py": test_conanfile,
                     'test_package/CMakeLists.txt': test_cmake,
                     "test_package/timer.cpp": self.timer_cpp})
        client.run("create . %s" % settings)
        if not len(settings):
            self.assertIn("Hello World Release!", client.out)

    @pytest.mark.tool_cmake(version="3.19")
    def test_apple_own_framework_cmake_multi(self):
        client = TestClient()

        test_cmake = textwrap.dedent("""
            project(Testing CXX)
            message(STATUS "CMAKE_BINARY_DIR ${CMAKE_BINARY_DIR}")
            include(${CMAKE_BINARY_DIR}/../../conanbuildinfo_multi.cmake)
            conan_basic_setup()
            message(">>> CONAN_FRAMEWORKS_FOUND_MYLIBRARY_DEBUG: ${CONAN_FRAMEWORKS_FOUND_MYLIBRARY_DEBUG}")
            message(">>> CONAN_FRAMEWORKS_FOUND_MYLIBRARY_RELEASE: ${CONAN_FRAMEWORKS_FOUND_MYLIBRARY_RELEASE}")
            add_executable(timer timer.cpp)
            target_link_libraries(timer debug ${CONAN_LIBS_DEBUG} optimized ${CONAN_LIBS_RELEASE})
        """)

        test_conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake
            class TestPkg(ConanFile):
                generators = "cmake_multi"
                name = "app"
                version = "1.0"
                requires = "mylibrary/1.0"
                exports_sources = "CMakeLists.txt", "timer.cpp"
                settings = "build_type",  # cmake_multi doesn't work without build_type
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                def test(self):
                    self.run("%s/timer" % self.settings.build_type, run_environment=True)
            """)
        client.save({'conanfile.py': self.conanfile,
                     "src/CMakeLists.txt": self.cmake,
                     "src/hello.h": self.hello_h,
                     "src/hello.cpp": self.hello_cpp,
                     "src/Info.plist": self.infoplist})
        client.run("export . mylibrary/1.0@")
        client.run("create . mylibrary/1.0@ -s build_type=Debug")
        client.run("create . mylibrary/1.0@ -s build_type=Release")

        client.save({"conanfile.py": test_conanfile,
                     'CMakeLists.txt': test_cmake,
                     "timer.cpp": self.timer_cpp})
        with environment_append({"CONAN_CMAKE_GENERATOR": "Xcode"}):
            client.run("install . -s build_type=Debug")
            client.run("install . -s build_type=Release")
            client.run("test . mylibrary/1.0@")
            self.assertIn("Hello World Release!", client.out)
            client.run("test . mylibrary/1.0@ -s build_type=Debug")
            self.assertIn("Hello World Debug!", client.out)

    def test_apple_own_framework_cmake_find_package_multi(self):
        client = TestClient()

        test_cmake = textwrap.dedent("""
            project(Testing CXX)
            set(CMAKE_RUNTIME_OUTPUT_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}/bin")
            set(CMAKE_RUNTIME_OUTPUT_DIRECTORY_DEBUG "${CMAKE_CURRENT_BINARY_DIR}/bin")
            set(CMAKE_RUNTIME_OUTPUT_DIRECTORY_RELEASE "${CMAKE_CURRENT_BINARY_DIR}/bin")
            find_package(mylibrary REQUIRED)
            message(">>> CONAN_FRAMEWORKS_FOUND_MYLIBRARY: ${CONAN_FRAMEWORKS_FOUND_MYLIBRARY}")
            add_executable(timer timer.cpp)
            target_link_libraries(timer mylibrary::mylibrary)
        """)

        test_conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake
            class TestPkg(ConanFile):
                generators = "cmake_find_package_multi"
                settings = "build_type",
                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                def test(self):
                    self.run("bin/timer", run_environment=True)
            """)
        client.save({'conanfile.py': self.conanfile,
                     "src/CMakeLists.txt": self.cmake,
                     "src/hello.h": self.hello_h,
                     "src/hello.cpp": self.hello_cpp,
                     "src/Info.plist": self.infoplist,
                     "test_package/conanfile.py": test_conanfile,
                     'test_package/CMakeLists.txt': test_cmake,
                     "test_package/timer.cpp": self.timer_cpp})
        client.run("create .")
        self.assertIn("Hello World Release!", client.out)

    @parameterized.expand([('cmake', False),
                           ('cmake_find_package', False), ('cmake_find_package', True), ])
    def test_frameworks_exelinkflags(self, generator, use_components):
        # FIXME: Conan 2.0. 'cpp_info' object has a 'frameworks' key
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                options = {'use_components': [True, False]}
                default_options = {'use_components': False}

                def package_info(self):
                    if not self.options.use_components:
                        self.cpp_info.exelinkflags.extend(['-framework Foundation'])
                        #self.cpp_info.frameworks.extend(['Foundation'])
                    else:
                        self.cpp_info.components["cmp"].exelinkflags.extend(['-framework Foundation'])
                        #self.cpp_info.components["cmp"].frameworks.extend(['Foundation'])
        """)
        tp_cmakelists = textwrap.dedent("""
            cmake_minimum_required(VERSION 3.0)
            project(test_package)

            include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
            conan_basic_setup(TARGETS)

            if(USE_FIND_PACKAGE)
                message(">> USE_FIND_PACKAGE")
                find_package(name)
                add_executable(${PROJECT_NAME} test_package.cpp)
                if (USE_COMPONENTS)
                    message(">> USE_COMPONENTS")
                    target_link_libraries(${PROJECT_NAME} name::cmp)
                else()
                    message(">> not USE_COMPONENTS")
                    target_link_libraries(${PROJECT_NAME} name::name)
                endif()
            else()
                message(">> not USE_FIND_PACKAGE")
                add_executable(${PROJECT_NAME} test_package.cpp)
                target_link_libraries(${PROJECT_NAME} CONAN_PKG::name)
            endif()
        """)
        tp_conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake

            class TestPackage(ConanFile):
                settings = "os", "arch", "compiler", "build_type"
                generators = "cmake", "cmake_find_package"
                options = {'use_find_package': [True, False]}
                requires = "name/version"

                def build(self):
                    cmake = CMake(self)
                    cmake.definitions["USE_FIND_PACKAGE"] = self.options.use_find_package
                    cmake.definitions["USE_COMPONENTS"] = self.options["name"].use_components
                    cmake.configure()
                    cmake.build()

                def test(self):
                    pass
        """)
        tp_main = textwrap.dedent("""
            int main() {}
        """)

        t = TestClient()
        t.save({'conanfile.py': conanfile,
                'test_package/conanfile.py': tp_conanfile,
                'test_package/CMakeLists.txt': tp_cmakelists,
                'test_package/test_package.cpp': tp_main})
        t.run("export conanfile.py name/version@")

        with t.chdir('test_package/build'):
            if generator == 'cmake':
                assert not use_components
                t.run("install .. --build=missing"
                      " -o name:use_components=False -o use_find_package=False")
                t.run("build ..")
                self.assertIn(">> not USE_FIND_PACKAGE", t.out)
            else:
                assert generator == 'cmake_find_package'
                t.run("install .. --build=missing"
                      " -o name:use_components={} -o use_find_package=True".format(use_components))
                t.run("build ..")
                self.assertIn(">> USE_FIND_PACKAGE", t.out)
                self.assertIn(">> {}USE_COMPONENTS".format("" if use_components else "not "), t.out)

            # Check we are using the framework
            link_txt = t.load(os.path.join('CMakeFiles', 'test_package.dir', 'link.txt'))
            self.assertIn("-framework Foundation", link_txt)

    def test_component(self):
        conanfile_py = textwrap.dedent("""
from conans import ConanFile, CMake, tools


class HelloConan(ConanFile):
    name = "hello"
    description = "example"
    topics = ("conan",)
    url = "https://github.com/conan-io/conan-center-index"
    homepage = "https://www.example.com"
    license = "MIT"
    exports_sources = ["hello.cpp", "hello.h", "CMakeLists.txt"]
    generators = "cmake"
    settings = "os", "arch", "compiler", "build_type"
    _source_subfolder = "source_subfolder"
    _build_subfolder = "build_subfolder"

    _cmake = None

    def source(self):
        pass

    def _configure_cmake(self):
        if self._cmake:
            return self._cmake
        self._cmake = CMake(self)
        self._cmake.configure(build_folder=self._build_subfolder)
        return self._cmake

    def build(self):
        cmake = self._configure_cmake()
        cmake.build()

    def package(self):
        cmake = self._configure_cmake()
        cmake.install()

    def package_info(self):
        self.cpp_info.names["cmake_find_package"] = "HELLO"
        self.cpp_info.names["cmake_find_package_multi"] = "HELLO"
        self.cpp_info.components["libhello"].names["cmake_find_package"] = "libhello"
        self.cpp_info.components["libhello"].names["cmake_find_package_multi"] = "libhello"

        self.cpp_info.components["libhello"].libs = ["hello"]
        self.cpp_info.components["libhello"].frameworks.extend(["CoreFoundation"])
        """)
        hello_cpp = textwrap.dedent("""
#include <CoreFoundation/CoreFoundation.h>

void hello_api()
{
    CFTypeRef keys[] = {CFSTR("key")};
    CFTypeRef values[] = {CFSTR("value")};
    CFDictionaryRef dict = CFDictionaryCreate(kCFAllocatorDefault, keys, values, sizeof(keys) / sizeof(keys[0]), &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
    if (dict)
        CFRelease(dict);
}
        """)
        hello_h = textwrap.dedent("""
void hello_api();
        """)
        cmakelists_txt = textwrap.dedent("""
cmake_minimum_required(VERSION 2.8)

project(hello)

include(GNUInstallDirs)

include(conanbuildinfo.cmake)
conan_basic_setup()

file(GLOB SOURCES *.cpp)
file(GLOB HEADERS *.h)

add_library(${PROJECT_NAME} ${SOURCES} ${HEADERS})

set_target_properties(${PROJECT_NAME} PROPERTIES PUBLIC_HEADER ${HEADERS})
install(TARGETS ${PROJECT_NAME}
    RUNTIME DESTINATION bin
    LIBRARY DESTINATION lib
    ARCHIVE DESTINATION lib
    PUBLIC_HEADER DESTINATION include)
        """)
        tp_conanfile_py = textwrap.dedent("""
import os
from conans import ConanFile, CMake, tools

class TestPackageConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "cmake", "cmake_find_package_multi"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def test(self):
        if not tools.cross_building(self.settings):
            bin_path = os.path.join("bin", "test_package")
            self.run(bin_path, run_environment=True)
        """)
        tp_test_package_cpp = textwrap.dedent("""
#include "hello.h"

int main()
{
    hello_api();
}
        """)
        tp_cmakelists_txt = textwrap.dedent("""
cmake_minimum_required(VERSION 2.8)
project(test_package)

include(${CMAKE_BINARY_DIR}/conanbuildinfo.cmake)
conan_basic_setup()

find_package(HELLO REQUIRED CONFIG)

add_executable(${PROJECT_NAME} test_package.cpp)
target_link_libraries(${PROJECT_NAME} HELLO::libhello)
        """)
        t = TestClient()
        t.save({'conanfile.py': conanfile_py,
                'hello.cpp': hello_cpp,
                'hello.h': hello_h,
                'CMakeLists.txt': cmakelists_txt,
                'test_package/conanfile.py': tp_conanfile_py,
                'test_package/CMakeLists.txt': tp_cmakelists_txt,
                'test_package/test_package.cpp': tp_test_package_cpp})
        t.run("create . hello/1.0@")



# coding=utf-8

import json
import os
import textwrap
import unittest

from conans.client.build.cppstd_flags import cppstd_default
from conans.client.conf import get_default_settings_yml
from conans.client.tools import environment_append, save, load
from conans.model.settings import Settings
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient


class DefaultCppTestCase(unittest.TestCase):
    compiler = "gcc"
    compiler_version = "7"

    default_profile = textwrap.dedent("""
        [settings]
        os=Linux
        arch=x86
        compiler={}
        compiler.version={}
        compiler.libcxx=libstdc++11
        """.format(compiler, compiler_version))

    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Library(ConanFile):
            settings = "{settings}"

            def configure(self):
                cppstd = self.settings.get_safe("cppstd")
                compiler_cppstd = self.settings.get_safe("compiler.cppstd")
                self.output.info(">>>> settings: {{}}".format(self.settings.fields))
                self.output.info(">>>> cppstd: {{}}".format(cppstd))
                self.output.info(">>>> compiler.cppstd: {{}}".format(compiler_cppstd))
        """)

    id_default = "d17189cfe7b11efbc5d701339a32d203745f8b81"

    def run(self, *args, **kwargs):
        default_profile_path = os.path.join(temp_folder(), "default.profile")
        save(default_profile_path, self.default_profile)
        with environment_append({"CONAN_DEFAULT_PROFILE_PATH": default_profile_path}):
            super(DefaultCppTestCase, self).run(*args, **kwargs)

    def setUp(self):
        self.t = TestClient()
        # Compute ID without the setting 'cppstd'
        target_id, output = self._get_id(with_cppstd=False)
        self.assertEqual(target_id, self.id_default)
        self.assertIn(">>>> settings: ['compiler', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: None", output)

    def _get_id(self, with_cppstd, settings_values=None):
        # Create the conanfile with corresponding settings
        settings = ["os", "compiler", ]
        if with_cppstd:
            settings += ["cppstd"]
        conanfile = self.conanfile.format(settings='", "'.join(settings))
        self.t.save({"conanfile.py": conanfile}, clean_first=True)

        # Create the string with the command line settings
        settings_values = settings_values or dict()
        settings_values_str = ["-s {k}={v}".format(k=k, v=v) for k, v in settings_values.items()]
        settings_values_str = " ".join(settings_values_str)

        # Call `conan info`
        json_file = os.path.join(self.t.current_folder, "tmp.json")
        self.t.run('info . {} --json="{}"'.format(settings_values_str, json_file))
        info_output = self.t.out
        data = json.loads(load(json_file))
        self.assertEqual(len(data), 1)

        # Return ID, output
        return data[0]["id"], info_output


def _make_cppstd_default(compiler, compiler_version):
    settings = Settings.loads(get_default_settings_yml())
    settings.compiler = compiler
    settings.compiler.version = compiler_version
    return cppstd_default(settings)


class SettingsCppStdTests(DefaultCppTestCase):
    """
    Validate package ID computed taking into account different scenarios for 'cppstd'. The ID
    should be the same if the setting is not provided and if it has the default value.
    """

    def test_no_value(self):
        # No value passed for setting 'cppstd'
        id_with, output = self._get_id(with_cppstd=True)  # TODO: Should raise?
        self.assertIn(">>>> settings: ['compiler', 'cppstd', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: None", output)
        self.assertEqual(self.id_default, id_with)

    def test_value_none(self):
        # Explicit value 'None' passed to setting 'cppstd'
        id_with, output = self._get_id(with_cppstd=True, settings_values={"cppstd": "None"})
        self.assertIn(">>>> settings: ['compiler', 'cppstd', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: None", output)
        self.assertEqual(self.id_default, id_with)

    def test_value_default(self):
        # Explicit value (equals to default) passed to setting 'cppstd'
        cppstd = _make_cppstd_default(self.compiler, self.compiler_version)
        id_with, output = self._get_id(with_cppstd=True, settings_values={"cppstd": cppstd})
        self.assertIn(">>>> settings: ['compiler', 'cppstd', 'os']", output)
        self.assertIn(">>>> cppstd: gnu14", output)
        self.assertIn(">>>> compiler.cppstd: None", output)
        self.assertEqual(self.id_default, id_with)

    def test_value_non_default(self):
        # Explicit value (not the default) passed to setting 'cppstd'
        id_with, output = self._get_id(with_cppstd=True, settings_values={"cppstd": "14"})
        self.assertIn(">>>> settings: ['compiler', 'cppstd', 'os']", output)
        self.assertIn(">>>> cppstd: 14", output)
        self.assertIn(">>>> compiler.cppstd: None", output)
        self.assertNotEqual(self.id_default, id_with)


class SettingsCompilerCppStdTests(DefaultCppTestCase):
    """
    Validate package ID computed taking into account different scenarios for 'compiler.cppstd'. The
    ID has to be the same if the setting is not informed and if it has the default value, also
    these values should be the same as the ones using the 'cppstd' approach.
    """

    def _get_id(self, with_cppstd=False, settings_values=None):
        assert not with_cppstd
        return super(SettingsCompilerCppStdTests, self)._get_id(with_cppstd=False,
                                                                settings_values=settings_values)

    def test_value_none(self):
        # Explicit value 'None' passed to setting 'cppstd'
        id_with, output = self._get_id(settings_values={"compiler.cppstd": "None"})
        self.assertIn(">>>> settings: ['compiler', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: None", output)
        self.assertEqual(self.id_default, id_with)

    def test_value_default(self):
        # Explicit value (equals to default) passed to setting 'compiler.cppstd'
        cppstd = _make_cppstd_default(self.compiler, self.compiler_version)
        id_with, output = self._get_id(settings_values={"compiler.cppstd": cppstd})
        self.assertIn(">>>> settings: ['compiler', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: gnu14", output)
        self.assertEqual(self.id_default, id_with)

    def test_value_other(self):
        # Explicit value (not the default) passed to setting 'cppstd'
        id_with, output = self._get_id(settings_values={"compiler.cppstd": "14"})
        self.assertIn(">>>> settings: ['compiler', 'os']", output)
        self.assertIn(">>>> cppstd: None", output)
        self.assertIn(">>>> compiler.cppstd: 14", output)
        self.assertNotEqual(self.id_default, id_with)


class SettingsCompareCppStdApproaches(DefaultCppTestCase):
    """
    Check scenario using 'cppstd' and 'compiler.cppstd', if those are given the same value
    (but different from the default one) then the ID for the packages is not required to be
    the same.
    """

    def test_cppstd_non_defaults(self):
        cppstd_value = "14"  # Not the default
        id_with_old, _ = self._get_id(with_cppstd=True, settings_values={"cppstd": cppstd_value})
        id_with_new, _ = self._get_id(with_cppstd=False,
                                      settings_values={'compiler.cppstd': cppstd_value})

        # Those are different from the target one (ID using default value or None)
        self.assertNotEqual(self.id_default, id_with_old)
        self.assertNotEqual(self.id_default, id_with_new)

        # They are different between them
        self.assertNotEqual(id_with_new, id_with_old)



import os
import platform
import textwrap
import unittest

import pytest
import six

from conans.client.runner import ConanRunner
from conans.client.tools import environment_append
from conans.test.utils.mocks import TestBufferConanOutput
from conans.test.utils.tools import TestClient


class RunnerTest(unittest.TestCase):

    def _install_and_build(self, conanfile_text, runner=None):
        client = TestClient(runner=runner)
        files = {"conanfile.py": conanfile_text}
        test_folder = os.path.join(client.current_folder, "test_folder")
        self.assertFalse(os.path.exists(test_folder))
        client.save(files)
        client.run("install .")
        client.run("build .")
        return client

    def test_ignore_error(self):
        conanfile = """from conans import ConanFile
class Pkg(ConanFile):
    def source(self):
        ret = self.run("not_a_command", ignore_errors=True)
        self.output.info("RETCODE %s" % (ret!=0))
"""
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        self.assertIn("RETCODE True", client.out)

    def test_basic(self):
        conanfile = '''
from conans import ConanFile
from conans.client.runner import ConanRunner
import platform

class ConanFileToolsTest(ConanFile):

    def build(self):
        self._runner = ConanRunner()
        self.run("mkdir test_folder")
    '''
        client = self._install_and_build(conanfile)
        test_folder = os.path.join(client.current_folder, "test_folder")
        self.assertTrue(os.path.exists(test_folder))

    def test_write_to_stringio(self):
        runner = ConanRunner(print_commands_to_output=True,
                             generate_run_log_file=True,
                             log_run_to_output=True)

        out = six.StringIO()
        runner("python --version", output=out)
        self.assertIn("""---Running------
> python --version
-----------------""", out.getvalue())

    @pytest.mark.tool_cmake
    def test_log(self):
        conanfile = '''
from conans import ConanFile

class ConanFileToolsTest(ConanFile):
    def build(self):
        self.run("cmake --version")
'''
        # A runner logging everything
        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=True,
                             generate_run_log_file=True,
                             log_run_to_output=True,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertIn("--Running---", output)
        self.assertIn("> cmake --version", output)
        self.assertIn("cmake version", output)
        self.assertIn("Logging command output to file ", output)

        # A runner logging everything
        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=True,
                             generate_run_log_file=False,
                             log_run_to_output=True,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertIn("--Running---", output)
        self.assertIn("> cmake --version", output)
        self.assertIn("cmake version", output)
        self.assertNotIn("Logging command output to file ", output)

        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=False,
                             generate_run_log_file=True,
                             log_run_to_output=True,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertNotIn("--Running---", output)
        self.assertNotIn("> cmake --version", output)
        self.assertIn("cmake version", output)
        self.assertIn("Logging command output to file ", output)

        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=False,
                             generate_run_log_file=False,
                             log_run_to_output=True,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertNotIn("--Running---", output)
        self.assertNotIn("> cmake --version", output)
        self.assertIn("cmake version", output)
        self.assertNotIn("Logging command output to file ", output)

        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=False,
                             generate_run_log_file=False,
                             log_run_to_output=False,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertNotIn("--Running---", output)
        self.assertNotIn("> cmake --version", output)
        self.assertNotIn("cmake version", output)
        self.assertNotIn("Logging command output to file ", output)

        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=False,
                             generate_run_log_file=True,
                             log_run_to_output=False,
                             output=output)
        self._install_and_build(conanfile, runner=runner)
        self.assertNotIn("--Running---", output)
        self.assertNotIn("> cmake --version", output)
        self.assertNotIn("cmake version", output)
        self.assertIn("Logging command output to file ", output)

    def test_cwd(self):
        conanfile = '''
from conans import ConanFile
from conans.client.runner import ConanRunner
import platform

class ConanFileToolsTest(ConanFile):

    def build(self):
        self._runner = ConanRunner()
        self.run("mkdir test_folder", cwd="child_folder")
    '''
        files = {"conanfile.py": conanfile}

        client = TestClient()
        os.makedirs(os.path.join(client.current_folder, "child_folder"))
        test_folder = os.path.join(client.current_folder, "child_folder", "test_folder")
        self.assertFalse(os.path.exists(test_folder))
        client.save(files)
        client.run("install .")
        client.run("build .")
        self.assertTrue(os.path.exists(test_folder))

    def test_cwd_error(self):
        conanfile = '''
from conans import ConanFile
from conans.client.runner import ConanRunner
import platform

class ConanFileToolsTest(ConanFile):

    def build(self):
        self._runner = ConanRunner()
        self.run("mkdir test_folder", cwd="non_existing_folder")
    '''
        files = {"conanfile.py": conanfile}

        client = TestClient()
        test_folder = os.path.join(client.current_folder, "child_folder", "test_folder")
        self.assertFalse(os.path.exists(test_folder))
        client.save(files)
        client.run("install .")
        client.run("build .", assert_error=True)
        self.assertIn("Error while executing 'mkdir test_folder'", client.out)
        self.assertFalse(os.path.exists(test_folder))

    def test_runner_capture_output(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                def source(self):
                    self.run("echo 'hello Conan!'")
        """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        self.assertIn("hello Conan!", client.out)

    def test_custom_stream_error(self):
        # https://github.com/conan-io/conan/issues/7888
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                def source(self):
                    class Buf:
                        def __init__(self):
                            self.buf = []

                        def write(self, data):
                            self.buf.append(data)

                    my_buf = Buf()
                    self.run('echo "Hello"', output=my_buf)
                    self.output.info("Buffer got msgs {}".format(len(my_buf.buf)))
            """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        self.assertIn("Buffer got msgs 1", client.out)

    def test_credentials_removed(self):
        conanfile = textwrap.dedent("""
            import os
            import platform
            from conans import ConanFile

            class Recipe(ConanFile):
                def export(self):
                    self.output.info(">> key: {}<<".format(os.getenv('CONAN_LOGIN_ENCRYPTION_KEY')))
                    self.output.info(">> var: {}<<".format(os.getenv('OTHER_VAR')))
                    if platform.system() == 'Windows':
                        self.run("echo key: %CONAN_LOGIN_ENCRYPTION_KEY%--")
                        self.run("echo var: %OTHER_VAR%--")
                    else:
                        self.run("echo key: $CONAN_LOGIN_ENCRYPTION_KEY--")
                        self.run("echo var: $OTHER_VAR--")
        """)
        with environment_append({'CONAN_LOGIN_ENCRYPTION_KEY': 'secret!', 'OTHER_VAR': 'other_var'}):
            client = TestClient()
            client.save({"conanfile.py": conanfile})
            client.run("export . name/version@")
            self.assertIn("name/version: >> key: secret!<<", client.out)
            self.assertIn("name/version: >> var: other_var<<", client.out)
            if platform.system() == 'Windows':
                self.assertIn("key: %CONAN_LOGIN_ENCRYPTION_KEY%--", client.out)
            else:
                self.assertIn("key: --", client.out)
            self.assertIn("var: other_var--", client.out)



# coding=utf-8

import unittest
from collections import namedtuple
from conans.client import rest


class RestStringTest(unittest.TestCase):

    def _get_html_response(self):
        headers = {"content-type": "text/html;charset=utf-8",}
        return namedtuple("Response", "status_code headers reason content")(
            404,
            headers,
            "Not Found",
            b'\n<!doctype html>\n<!--[if IE 9]><html class="lt-ie10" lang="en" >\n</html>\n')

    def _get_json_response(self):
        headers = {"content-type": "application/json"}
        return namedtuple("Response", "status_code headers reason content")(
            404,
            headers,
            "Not Found",
            b'Could not find the artifact')

    def test_html_error(self):
        response = self._get_html_response()
        result = rest.response_to_str(response)
        self.assertEqual("404: Not Found", result)

    def test_json_error(self):
        response = self._get_json_response()
        result = rest.response_to_str(response)
        self.assertEqual("Could not find the artifact", result)



import unittest

from conans.client.conf import get_default_settings_yml
from conans.client.generators.compiler_args import CompilerArgsGenerator
from conans.client.generators.gcc import GCCGenerator
from conans.model.build_info import CppInfo, DepsCppInfo
from conans.model.env_info import DepsEnvInfo, EnvInfo
from conans.model.settings import Settings
from conans.model.user_info import DepsUserInfo
from conans.test.utils.mocks import ConanFileMock


class CompilerArgsTest(unittest.TestCase):

    def test_visual_studio_extensions(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.build_type = "Release"

        conan_file = ConanFileMock()
        conan_file.settings = settings
        conan_file.deps_env_info = DepsEnvInfo()
        conan_file.deps_user_info = DepsUserInfo()
        conan_file.deps_cpp_info = DepsCppInfo()
        cpp_info = CppInfo("zlib", "/root")
        cpp_info.libs.append("mylib")
        cpp_info.libs.append("other.lib")
        conan_file.deps_cpp_info.add("zlib", cpp_info)
        conan_file.env_info = EnvInfo()

        gen = CompilerArgsGenerator(conan_file)
        self.assertEqual('-O2 -Ob2 -DNDEBUG -link mylib.lib other.lib', gen.content)

    @staticmethod
    def _get_conanfile(settings, frameworks=False, system_libs=False):
        conan_file = ConanFileMock()
        conan_file.settings = settings
        conan_file.folders.set_base_source("my_cache_source_folder")
        conan_file.folders.set_base_build("my_cache_build_folder")
        conan_file.deps_env_info = DepsEnvInfo()
        conan_file.deps_user_info = DepsUserInfo()
        conan_file.deps_cpp_info = DepsCppInfo()

        cpp_info = CppInfo("zlib", "/root")
        cpp_info.includedirs.append("path/to/include1")
        cpp_info.libdirs.append("path/to/lib1")
        cpp_info.libs.append("mylib")
        cpp_info.bindirs = "path/to/bin1"
        cpp_info.cflags.append("c_flag1")
        cpp_info.cxxflags.append("cxx_flag1")
        cpp_info.defines.append("mydefine1")
        if system_libs:
            cpp_info.system_libs.append("system_lib1")
        if frameworks:
            cpp_info.frameworks = ["AVFoundation", "VideoToolbox"]
            cpp_info.frameworkdirs.extend(['path/to/Frameworks1', 'path/to/Frameworks2'])
        cpp_info.filter_empty = False
        conan_file.deps_cpp_info.add("zlib", cpp_info)

        conan_file.env_info = EnvInfo()
        return conan_file

    def test_gcc(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "6.3"
        settings.arch = "x86"
        settings.build_type = "Release"
        settings.cppstd = "gnu17"

        conan_file = self._get_conanfile(settings)
        gcc = GCCGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m32 -O3 -s -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -F /root/Frameworks -std=gnu++17', gcc.content)

        settings.arch = "x86_64"
        settings.build_type = "Debug"
        settings.compiler.libcxx = "libstdc++11"

        gcc = GCCGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m64 -g'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -D_GLIBCXX_USE_CXX11_ABI=1 -F /root/Frameworks -std=gnu++17',
                         gcc.content)

        settings.compiler.libcxx = "libstdc++"
        gcc = GCCGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m64 -g'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -D_GLIBCXX_USE_CXX11_ABI=0 -F /root/Frameworks -std=gnu++17',
                         gcc.content)

        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.build_type = "Release"
        gcc = GCCGenerator(conan_file)
        # GCC generator ignores the compiler setting, it is always gcc
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m32 -O3 -s -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -D_GLIBCXX_USE_CXX11_ABI=0 -F /root/Frameworks -std=gnu++17',
                         gcc.content)

    def test_compiler_args(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.build_type = "Release"

        conan_file = self._get_conanfile(settings)
        gen = CompilerArgsGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I\\root\\include -I\\root\\path\\to\\include1'
                         ' cxx_flag1 c_flag1 -O2 -Ob2 -DNDEBUG -link'
                         ' -LIBPATH:\\root\\lib -LIBPATH:\\root\\path\\to\\lib1 mylib.lib',
                         gen.content)

        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Macos"
        settings.compiler = "apple-clang"
        settings.compiler.version = "9.0"
        settings.arch = "x86"
        settings.build_type = "Release"
        conan_file = self._get_conanfile(settings)
        gen = CompilerArgsGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m32 -O3 -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -F /root/Frameworks', gen.content)

        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.os_build = "Macos"
        settings.compiler = "apple-clang"
        settings.compiler.version = "9.0"
        settings.arch = "x86"
        settings.build_type = "Release"

        conan_file = self._get_conanfile(settings)
        args = CompilerArgsGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m32 -O3 -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -F /root/Frameworks', args.content)

    def test_apple_frameworks(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Macos"
        settings.compiler = "apple-clang"
        settings.compiler.version = "9.1"
        settings.arch = "x86_64"
        settings.build_type = "Release"

        conan_file = self._get_conanfile(settings, frameworks=True)
        args = CompilerArgsGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m64 -O3 -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib'
                         ' -framework AVFoundation -framework VideoToolbox'
                         ' -F /root/Frameworks -F /root/path/to/Frameworks1'
                         ' -F /root/path/to/Frameworks2', args.content)

    def test_system_libs(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Linux"
        settings.compiler = "gcc"
        settings.compiler.version = "8"
        settings.arch = "x86_64"
        settings.build_type = "Release"

        conan_file = self._get_conanfile(settings, system_libs=True)
        args = CompilerArgsGenerator(conan_file)
        self.assertEqual('-Dmydefine1 -I/root/include -I/root/path/to/include1'
                         ' cxx_flag1 c_flag1 -m64 -O3 -s -DNDEBUG'
                         ' -Wl,-rpath,"/root/lib" -Wl,-rpath,"/root/path/to/lib1"'
                         ' -L/root/lib -L/root/path/to/lib1 -lmylib -lsystem_lib1'
                         ' -F /root/Frameworks', args.content)



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class UserInfoTestCase(unittest.TestCase):
    """ When using several contexts (xbuild feature) the user information from each of the
        contexts should be accessible, Conan will provide 'deps_user_info' attribute to access
        information from the 'host' context (like it is doing with 'deps_cpp_info'), but
        information from other contexts will be accessible via attributes 'user_info_<context>'.
    """

    library = textwrap.dedent("""
        from conans import ConanFile

        class Library(ConanFile):
            name = "library"
            settings = "os"

            def package_info(self):
                self.user_info.DATA = "{}-{}".format(self.name, self.settings.os)
    """)

    br = textwrap.dedent("""
        from conans import ConanFile

        class BuildRequires(ConanFile):
            settings = "os"

            requires = "library/1.0"

            def package_info(self):
                self.user_info.DATA = "{}-{}".format(self.name, self.settings.os)
    """)

    app = textwrap.dedent("""
        from conans import ConanFile

        class Library(ConanFile):
            name = "app"
            settings = "os"

            def requirements(self):
                self.requires("library/1.0")

            def build_requirements(self):
                self.build_requires("br_build/1.0")
                self.build_requires("br_host/1.0", force_host_context=True)

            def build(self):
                _info = self.output.info
                _info("[deps] {}".format(', '.join(sorted(self.deps_user_info.keys()))))
                _info("[deps] library.DATA={}".format(self.deps_user_info["library"].DATA))
                _info("[deps] br_host.DATA={}".format(self.deps_user_info["br_host"].DATA))

                _info("[build] {}".format(', '.join(sorted(self.user_info_build.keys()))))
                _info("[build] library.DATA={}".format(self.user_info_build["library"].DATA))
                _info("[build] br_build.DATA={}".format(self.user_info_build["br_build"].DATA))
    """)

    @classmethod
    def setUpClass(cls):
        super(UserInfoTestCase, cls).setUpClass()
        cls.t = TestClient()
        cls.t.save({'library.py': cls.library,
                    'build_requires.py': cls.br,
                    'app.py': cls.app,
                    'host': '[settings]\nos=Windows',
                    'build': '[settings]\nos=Linux', })
        cls.t.run("create library.py library/1.0@ --profile=host")
        cls.t.run("create library.py library/1.0@ --profile=build")
        cls.t.run("create build_requires.py br_host/1.0@ --profile=host")
        cls.t.run("create build_requires.py br_build/1.0@ --profile=build")

    def _check_user_info_data(self, app_scope, output):
        # Check information from the host context (using deps_user_info attribute)
        self.assertIn(app_scope + ": [deps] br_host, library", output)
        self.assertIn(app_scope + ": [deps] library.DATA=library-Windows", output)
        self.assertIn(app_scope + ": [deps] br_host.DATA=br_host-Windows", output)

        # Check information from the build context (using user_info_build attribute)
        self.assertIn(app_scope + ": [build] br_build, library", output)
        self.assertIn(app_scope + ": [build] library.DATA=library-Linux", output)
        self.assertIn(app_scope + ": [build] br_build.DATA=br_build-Linux", output)

    def test_user_info_from_requirements(self):
        self.t.run("create app.py app/1.0@ --profile:host=host --profile:build=build")
        self._check_user_info_data("app/1.0", self.t.out)

    def test_user_info_local_workflow(self):
        self.t.run("install app.py app/1.0@ --profile:host=host --profile:build=build")
        self.t.run("build app.py")
        self._check_user_info_data("app.py (app/1.0)", self.t.out)



import os
import platform
import textwrap

from conan.tools.files import load
from conans.test.assets.sources import gen_function_cpp
from conans.test.functional.toolchains.meson._base import TestMesonBase
from conans.test.utils.tools import TestClient


class TestMesonToolchainAndGnuFlags(TestMesonBase):

    def test_mesondeps(self):
        client = TestClient(path_with_spaces=False)
        client.run("new hello/0.1 -s")
        client.run("create . hello/0.1@ %s" % self._settings_str)
        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import Meson

        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello/0.1"
            generators = "MesonDeps", "MesonToolchain"

            def layout(self):
                self.folders.build = "build"

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build()
        """)

        meson_build = textwrap.dedent("""
        project('tutorial', 'cpp')
        cxx = meson.get_compiler('cpp')
        hello = cxx.find_library('hello', required: true)
        executable('demo', 'main.cpp', dependencies: hello)
        """)

        client.save({"conanfile.py": conanfile_py,
                     "meson.build": meson_build,
                     "main.cpp": app},
                    clean_first=True)

        client.run("install . %s" % self._settings_str)
        client.run("build .")
        assert "[2/2] Linking target demo" in client.out

    def test_mesondeps_flags_are_being_appended_and_not_replacing_toolchain_ones(self):
        """
        Test MesonDeps and MesonToolchain are keeping all the flags/definitions defined
        from both generators and nothing is being messed up.
        """
        client = TestClient(path_with_spaces=False)
        if platform.system() == "Windows":
            deps_flags = '"/GA", "/analyze:quiet"'
            flags = '"/Wall", "/W4"'
        else:
            deps_flags = '"-Wpedantic", "-Werror"'
            flags = '"-Wall", "-finline-functions"'
        # Dependency - hello/0.1
        conanfile_py = textwrap.dedent("""
        from conan import ConanFile

        class HelloConan(ConanFile):
            name = "hello"
            version = "0.1"

            def package_info(self):
                self.cpp_info.libs = ["hello"]
                self.cpp_info.cxxflags = [{}]
                self.cpp_info.defines = ['DEF1=one_string', 'DEF2=other_string']
        """.format(deps_flags))
        client.save({"conanfile.py": conanfile_py})
        client.run("create . %s" % self._settings_str)
        # Dependency - other/0.1
        conanfile_py = textwrap.dedent("""
        from conan import ConanFile

        class OtherConan(ConanFile):
            name = "other"
            version = "0.1"

            def package_info(self):
                self.cpp_info.libs = ["other"]
                self.cpp_info.defines = ['DEF3=simple_string']
        """)
        client.save({"conanfile.py": conanfile_py}, clean_first=True)
        client.run("create . %s" % self._settings_str)

        # Consumer using MesonDeps and MesonToolchain
        conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import Meson, MesonDeps, MesonToolchain

        class App(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            requires = "hello/0.1", "other/0.1"

            def layout(self):
                self.folders.build = "build"

            def generate(self):
                tc = MesonDeps(self)
                tc.generate()
                tc = MesonToolchain(self)
                tc.preprocessor_definitions["VAR"] = "VALUE"
                tc.preprocessor_definitions["VAR2"] = "VALUE2"
                tc.generate()

            def build(self):
                meson = Meson(self)
                meson.configure()
                meson.build()
        """)

        meson_build = textwrap.dedent("""
            project('tutorial', 'cpp')
            cxx = meson.get_compiler('cpp')
            # It's not needed to declare "hello/0.1" as a dependency, only interested in flags
            executable('demo', 'main.cpp')
        """)
        client.save({"conanfile.py": conanfile_py,
                     "meson.build": meson_build,
                     "main.cpp": "int main()\n{return 0;}\n"},
                    clean_first=True)

        client.run("install . %s -c 'tools.build:cxxflags=[%s]'" % (self._settings_str, flags))
        client.run("build .")
        deps_flags = deps_flags.replace('"', "").replace(",", "")
        flags = flags.replace('"', "").replace(",", "")
        meson_log_path = os.path.join(client.current_folder, "build", "meson-logs", "meson-log.txt")
        meson_log = load(None, meson_log_path)
        meson_log = meson_log.replace("\\", "/")
        assert "Build Options: " \
               "'--native-file {folder}/conan_meson_native.ini' " \
               "'--native-file {folder}/conan_meson_deps_flags.ini'" \
               "".format(folder=client.current_folder.replace("\\", "/")) in meson_log
        # Flags/Defines from deps and consumer are appearing in meson-log.txt as part
        # of the command-line
        assert '%s -DVAR="VALUE" -DVAR2="VALUE2" %s' % (flags, deps_flags) in meson_log
        assert '-DDEF3=simple_string -DDEF1=one_string -DDEF2=other_string' in meson_log



import os
import platform
import textwrap
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient
from conans.util.files import load


class SourceDirtyTest(unittest.TestCase):
    def test_keep_failing_source_folder(self):
        # https://github.com/conan-io/conan/issues/4025
        client = TestClient()
        conanfile = textwrap.dedent("""\
            from conans import ConanFile
            from conans.tools import save
            class Pkg(ConanFile):
                def source(self):
                    save("somefile.txt", "hello world!!!")
                    raise Exception("boom")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/1.0@user/channel", assert_error=True)
        self.assertIn("ERROR: pkg/1.0@user/channel: Error in source() method, line 6", client.out)
        ref = ConanFileReference.loads("pkg/1.0@user/channel")
        # Check that we can debug and see the folder
        self.assertEqual(load(os.path.join(client.cache.package_layout(ref).source(),
                                           "somefile.txt")),
                         "hello world!!!")
        client.run("create . pkg/1.0@user/channel", assert_error=True)
        self.assertIn("pkg/1.0@user/channel: Source folder is corrupted, forcing removal",
                      client.out)
        client.save({"conanfile.py": conanfile.replace("source(", "source2(")})
        client.run("create . pkg/1.0@user/channel")
        self.assertIn("pkg/1.0@user/channel: Source folder is corrupted, forcing removal",
                      client.out)
        # Check that it is empty
        self.assertEqual(os.listdir(os.path.join(client.cache.package_layout(ref).source())), [])


@pytest.mark.skipif(platform.system() != "Windows", reason="Needs windows for rmdir block")
class ExportDirtyTest(unittest.TestCase):
    """ Make sure than when the source folder becomes dirty, due to a export of
    a new recipe with a rmdir failure, or to an uncomplete execution of source(),
    it is marked as dirty and removed when necessary
    """

    def setUp(self):
        self.client = TestClient()
        self.client.save({"conanfile.py": GenConanfile().with_exports("main.cpp"),
                          "main.cpp": ""})
        self.client.run("create . pkg/0.1@user/stable")
        ref = ConanFileReference.loads("pkg/0.1@user/stable")
        source_path = self.client.cache.package_layout(ref).source()
        file_open = os.path.join(source_path, "main.cpp")

        self.f = open(file_open, 'wb')
        self.f.write(b"Hello world")

        self.client.save({"conanfile.py": GenConanfile().with_exports("main.cpp", "other.h"),
                          "main.cpp": ""})
        self.client.run("export . pkg/0.1@user/stable")
        self.assertIn("ERROR: Unable to delete source folder. "
                      "Will be marked as corrupted for deletion",
                      self.client.out)

        self.client.run("install pkg/0.1@user/stable --build", assert_error=True)
        self.assertIn("ERROR: Unable to remove source folder", self.client.out)

    def test_export_remove(self):
        # export is able to remove dirty source folders
        self.f.close()
        self.client.run("export . pkg/0.1@user/stable")
        self.assertIn("Source folder is corrupted, forcing removal", self.client.out)
        self.client.run("install pkg/0.1@user/stable --build")
        self.assertNotIn("WARN: Trying to remove corrupted source folder", self.client.out)

    def test_install_remove(self):
        # install is also able to remove dirty source folders
        # Now, release the handle to the file
        self.f.close()
        self.client.run("install pkg/0.1@user/stable --build")
        self.assertIn("WARN: Trying to remove corrupted source folder", self.client.out)



import os
import platform
import subprocess
import textwrap
import unittest

import pytest

from conans.test.functional.generators.virtualenv_test import _load_env_file
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient
from conans.util.files import decode_text, save


@pytest.mark.skipif(platform.system() != "Windows", reason="Only for Windows")
class VirtualenvWindowsBashTestCase(unittest.TestCase):
    """
    We are running the full example inside Bash (generation of environment files and activate/deactivate), so we need
    to use actual Conan recipes and run an example. We cannot use the same approach as in 'virtualenv_test.py', but
    we should test the same cases
    """

    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile, tools

        class Recipe(ConanFile):
            def build(self):
                tools.save("executable.exe", "echo EXECUTABLE IN PACKAGE!!")

            def package(self):
                self.copy("*.exe", dst="bin")

            def package_info(self):
                # Basic variable
                self.env_info.USER_VAR = r"some value with space and \\ (backslash)"
                self.env_info.ANOTHER = "data"

                # List variable
                self.env_info.WHATEVER = ["list", "other"]
                self.env_info.WHATEVER2.append("list")

                # List with spaces
                self.env_info.CFLAGS = ["cflags1", "cflags2"]

                # Add something to the path
                self.env_info.PATH.append(os.path.join(self.package_folder, "bin"))

    """)

    @pytest.mark.tool_conan
    def test_git_shell(self):
        test_folder = temp_folder(path_with_spaces=False)

        cache_folder = os.path.join(temp_folder(path_with_spaces=False), ".conan")
        t = TestClient(cache_folder=cache_folder)
        t.save({"conanfile.py": self.conanfile})
        t.run("create . name/version@")

        # Locate the Conan we are actually using (it should be the one in this commit)
        stdout, _ = subprocess.Popen(["where", "conan"], stdout=subprocess.PIPE).communicate()
        conan_path = decode_text(stdout).splitlines()[0]  # Get the first one (Windows returns all found)

        # All the commands are listed in a sh file:
        commands_file = os.path.join(test_folder, 'commands.sh')
        conan_path = os.path.dirname(conan_path).replace('\\', '/').replace('C:', '/c')
        conan_user_home = os.path.dirname(t.cache_folder).replace('\\', '/').replace('C:', '/c')
        save(commands_file, textwrap.dedent("""
            export USER_VAR=existing_value
            export ANOTHER=existing_value
            export WHATEVER=existing_value
            export WHATEVER2=existing_value
            export CFLAGS=existing_value

            export PATH={conan_path}:$PATH
            export CONAN_USER_HOME={conan_user_home}
            conan install name/version@ -g virtualenv -g virtualrunenv

            env > env_before.txt
            echo 'Start to find executable'
            echo __exec_pre_path__=$(which executable)
            . ./activate.sh
            env > env_activated.txt
            echo __exec_env_path__=$(which executable)
            executable
            . ./deactivate.sh
            echo __exec_post_path__=$(which executable)
            env > env_after.txt
        """.format(conan_path=conan_path, conan_user_home=conan_user_home)))

        cmd = r'C:\Windows\System32\cmd.exe /c ""C:\Program Files\Git\bin\sh.exe" --login -i "{}""'.format(commands_file)
        shell = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=test_folder)
        (stdout, stderr) = shell.communicate()
        stdout, stderr = decode_text(stdout), decode_text(stderr)

        env_before = _load_env_file(os.path.join(test_folder, "env_before.txt"))
        env_after = _load_env_file(os.path.join(test_folder, "env_after.txt"))
        self.assertDictEqual(env_before, env_after)  # Environment restored correctly

        # Environment once activated
        environment = _load_env_file(os.path.join(test_folder, "env_activated.txt"))

        # Test a basic variable
        self.assertEqual(environment["USER_VAR"], r"some value with space and \ (backslash)")
        self.assertEqual(environment["ANOTHER"], "data")

        # Test find program
        epaths = dict(line.split("=", 1) for line in reversed(stdout.splitlines()) if line.startswith("__exec_"))
        self.assertEqual(epaths["__exec_pre_path__"], "")
        self.assertEqual(epaths["__exec_post_path__"], "")
        self.assertTrue(len(epaths["__exec_env_path__"].strip()) > 0)
        self.assertIn("EXECUTABLE IN PACKAGE!!", stdout)

        # Test variable which is a list
        self.assertEqual(environment["WHATEVER"], "list:other:existing_value")
        self.assertEqual(environment["WHATEVER2"], "list:existing_value")

        # Variable: list with spaces
        self.assertEqual(environment["CFLAGS"], "cflags1 cflags2 existing_value")



import platform
import sys
import unittest

import pytest

from conans.test.utils.tools import TestClient


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() not in ("Darwin", "Windows", "Linux"),
                    reason="Not tested for not mainstream boring operating systems")
class TestMesonBase(unittest.TestCase):
    def setUp(self):
        self.t = TestClient()

    @property
    def _settings(self):
        interpreter_arch = platform.machine()
        if interpreter_arch in ["x86_64", "AMD64"]:
            host_arch = "x86_64"
        elif interpreter_arch in ["arm64", "aarch64", "ARM64"]:
            host_arch = "armv8"
        else:
            host_ach = interpreter_arch
        settings_macosx = {"compiler": "apple-clang",
                           "compiler.libcxx": "libc++",
                           "compiler.version": "13",
                           "arch": host_arch,
                           "build_type": "Release"}

        settings_windows = {"compiler": "Visual Studio",
                            "compiler.version": "15",
                            "compiler.runtime": "MD",
                            "arch": host_arch,
                            "build_type": "Release"}

        settings_linux = {"compiler": "gcc",
                          "compiler.version": "5",
                          "compiler.libcxx": "libstdc++",
                          "arch": host_arch,
                          "build_type": "Release"}

        return {"Darwin": settings_macosx,
                "Windows": settings_windows,
                "Linux": settings_linux}.get(platform.system())

    @property
    def _settings_str(self):
        return " ".join('-s %s="%s"' % (k, v) for k, v in self._settings.items() if v)

    def _check_binary(self):
        # FIXME: Some values are hardcoded to match the CI setup
        host_arch =  self.t.get_default_host_profile().settings['arch']
        arch_macro = {
            "gcc": {"armv8": "__aarch64__", "x86_64": "__x86_64__"},
            "msvc": {"armv8": "_M_ARM64", "x86_64": "_M_X64"}
        }
        if platform.system() == "Darwin":
            self.assertIn(f"main {arch_macro['gcc'][host_arch]} defined", self.t.out)
            self.assertIn("main __apple_build_version__", self.t.out)
            self.assertIn("main __clang_major__13", self.t.out)
            # TODO: check why __clang_minor__ seems to be not defined in XCode 12
            # commented while migrating to XCode12 CI
            # self.assertIn("main __clang_minor__0", self.t.out)
        elif platform.system() == "Windows":
            self.assertIn(f"main {arch_macro['msvc'][host_arch]} defined", self.t.out)
            self.assertIn("main _MSC_VER19", self.t.out)
            self.assertIn("main _MSVC_LANG2014", self.t.out)
        elif platform.system() == "Linux":
            self.assertIn(f"main {arch_macro['gcc'][host_arch]} defined", self.t.out)
            self.assertIn("main __GNUC__5", self.t.out)



import os
import unittest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


class RemoveEmptyDirsTest(unittest.TestCase):

    def test_basic(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("Hello", "0.1")})
        client.run("export . lasote/stable")
        path = os.path.join(client.storage_folder, "Hello/0.1/lasote/stable")
        self.assertTrue(os.path.exists(path))
        client.run("remove Hello* -f")
        path = os.path.join(client.storage_folder, "Hello")
        self.assertFalse(os.path.exists(path))

    def test_shared_folder(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("Hello", "0.1")})
        client.run("export . lasote/stable")
        path = os.path.join(client.storage_folder, "Hello/0.1/lasote/stable")
        self.assertTrue(os.path.exists(path))
        client.run("export . lasote2/stable")
        path = os.path.join(client.storage_folder, "Hello/0.1/lasote2/stable")
        self.assertTrue(os.path.exists(path))
        client.run("remove Hello/0.1@lasote/stable -f")
        path = os.path.join(client.storage_folder, "Hello/0.1/lasote")
        self.assertFalse(os.path.exists(path))
        path = os.path.join(client.storage_folder, "Hello/0.1")
        self.assertTrue(os.path.exists(path))
